C*******************************************************************
C
C     Include file "cokrig.inc"
C
C     File "cokrig.inc" is a common file to be used throughout
C     the cokriging subroutines and functions.
C
C     Variables in the parameter statement have the following
C     meaning so resetting them and recompiling the program is
C     needed to update the program for more parameters.
C
C     MAXV         Maximum number of variables.
C     MAXV3        Maximum number of variables + 3.
C     MAXN         Maximum number of nested variograms per variable.
C     MAXGN        Maximum number of variograms = MAXN*MAXV*(MAXV+1)/2.
C     MAXNDP       Maximum number of discrete points = NPX*NPY*NPZ.
C     IASIZE       Dimension of "A" matrix = (NTS)*(NTS+1)/2.
C     MAXNUM       Max # of holes allowed in the programs.
C     INHOLE       Max # of sampled points = MAXNUM*MAXV.
C     MAXCHL       Max # of holes in kriging nbhd.
C     NTS          Max # of samples in kriging nbhd = MAXCHL*MAXV.
C                  Length is one row in the variance-covariance matrix.
C     MTERM        Maximum number of basis functions/variable
C                  times the number of variables.  Limits NXCOL.
C     NBRHS        Size of arrays for solution of basis functions.
C                  Dimension is set to NTS*MTERM
C     MAXG         MTERM*(MTERM+1)/2.
C
C**********************************************************************
C
cclong  PARAMETER(MAXV=10,MAXNDP=64,IASIZE=21000,MAXV3=7,MAXVS=16,
cc     1 MAXNUM=1000,INHOLE=4000,MAXCHL=50,NTS=200,NCRHS=800,MAXN=5,
cc     2 MAXGN=50,MTERM=20,NBRHS=4000,MAXG=210)
cc      maxv3 = maxv + 3, according to the comments above
cc	nts = maxchl*maxv = 50*5 = 250
cc	iasize = nts*(nts+1)/2  (dimension of A)
cc	maxgn = maxn*maxv*(maxv+1)/2 = 5*10*11/2 = 275
cc	inhole = maxnum*maxv = 1000*10 = 10000
cc	nbrhs = nts*mterm = 250*20 = 5000
cc	
      PARAMETER(MAXV=7,MAXNDP=64,MAXCHL=50,maxbasis=20,MAXN=20,
     & MAXNUM=300,
     & NTS=MAXCHL*MAXV,IASIZE=(NTS)*(NTS+1)/2,MTERM=maxbasis*maxv,
     & INHOLE=MAXNUM*MAXV, MAXGN=MAXN*MAXV*(MAXV+1)/2,
     & NBRHS=NTS*MTERM, MAXG=mterm*(mterm+1)/2)
cc      PARAMETER(MAXV=10,MAXNDP=64,IASIZE=31375,MAXV3=13,MAXVS=16,
cc     1 MAXNUM=1000,INHOLE=10000,MAXCHL=50,NTS=250,NCRHS=800,MAXN=5,
cc     2 MAXGN=275,MTERM=20,NBRHS=5000,MAXG=210)
cclong
C
C     Description of variables passed in common block / TITLES /
C
C     TITLE     Descriptive title of run.
C     NAME(J)   Name of variable J.
C
      COMMON / TITLES / TITLE,NAME
      CHARACTER*80 TITLE
cclong      CHARACTER*15 NAME(MAXV)
      CHARACTER*15 NAME(MAXV+3)
C
C     Description of variables passed in common block / GAM /.
C
C     HVALUE    Maximum distance in easting direction
C               to be utilized to estimate the sill
C               component of each of the nested models.
C     NEST(J)   Number of nested variogram models
C               for variable J.
C     NESTP(J)  Pointer to one location before the
C               start of the list of pointers for
C               variable J.
C     SILL(J)   Total sill of nested model J.
C     NTYPCS(J) Type of cross variogram if applicable.
C               Set to 0 - Not relevant.
C               Set to 1 - Sum variable representation.
C               Set to 2 - Difference variable representation.
C               Set to 3 - Signed nested model.
C               Set to 4 - Sum - difference representation with
C                          model stored as one signed nested model.
C
C     Note:  For the following variables, the pointer to
C            the nested model parameters of model J for
C            variable NVA is ==> I = NESTP(NVA) + J.
C
C     NVTYPE(I) Model type code.  Ranges from 1 - 17.
C     NSIGN(I)  Sign associated with the nested model.
C     C(I)      Sill parameter.
C     AAXIS(I)  Major ellipsoid axis range parameter, a.
C     BAXIS(I)  Minor ellipsoid axis range parameter, b.
C     CAXIS(I)  Minor ellipsoid axis range parameter, c.
C     AFAOB(I)  Anisotropy affinity ratio for axes a/b.
C     AFAOC(I)  Anisotropy affinity ratio for axes a/c.
C     THETA(I)  Theta (aximuth) ellipsoid rotation angle.
C     STHETA(I) Sine of theta rotation angle.
C     CTHETA(I) Cosine of theta rotation angle.
C     BETA(I)   Beta (dip) ellipsoid rotation angle.
C     SBETA(I)  Sine of beta rotation angle.
C     CBETA(I)  Cosine of beta rotation angle.
C     II        Matrix entry for variograms and cross-variograms
C     NP        Pointer of nested model parameters

      COMMON / GAM / HVALUE,NEST,NESTP,SILL,NTYPCS,ITH,NP,
     1         NVTYPE,C,AAXIS,BAXIS,CAXIS,AFAOB,AFAOC,
     3         THETA,STHETA,CTHETA,BETA,SBETA,CBETA,NSIGN
      real*4   HVALUE, SILL(MAXG), C(MAXGN),AAXIS(MAXGN),
     1         BAXIS(MAXGN),CAXIS(MAXGN),AFAOB(MAXGN),AFAOC(MAXGN),
     2         THETA(MAXGN),STHETA(MAXGN),CTHETA(MAXGN),
     3         BETA(MAXGN),SBETA(MAXGN),CBETA(MAXGN) 
      INTEGER*4 NEST(MAXG), NESTP(MAXG), NTYPCS(MAXG), NVTYPE(MAXGN),
     1          NSIGN(MAXGN),ITH,NP
C
C
C     Description of variables passed in common block / BLK /
C
C     NPX       Number of discrete points generated in X coordinates.
C     NPY       Number of discrete points generated in Y coordinates.
C     NPZ       Number of discrete points generated in Z coordinates.
C     NDP       Total number of discrete points generated.
C     XLBL      Length of block in X direction.
C     YLBL      Length of block in Y direction.
C     ZLBL      Length of block in Z direction.
C     XDB(*)    X coordinate of discrete point *.
C     YDB(*)    Y coordinate of discrete point *.
C     ZDB(*)    Z coordinate of discrete point *.
C     XEAST     Easting of southwest corner crest of block network.
C     YNORTH    Northing of southwest corner crest of block network.
C     ZELEV     Elevation of sourthwest corner crest of block network.
C     NSE       Starting Easting block number to krige.
C     NFE       Ending Easting block number to krige.
C     NSN       Starting Northing block number to krige.
C     NFN       Ending Northing block number to krige.
C     NSB       Starting bench number to krige.
C     NFB       Ending bench number to krige.
C
      COMMON / BLK / NPX,NPY,NPZ,NDP,XLBL,YLBL,ZLBL,
     1               XDB(MAXNDP),YDB(MAXNDP),ZDB(MAXNDP),
     2               XEAST,YNORTH,ZELEV,NSE,NFE,NSN,NFN,
     3               NSB,NFB
      Integer*4      NPX,NPY,NPZ,NDP,NSE,NSN,NSB,NFE,NFN,NFB
      Real*4         XEAST,YNORTH,ZELEV,XLBL,YLBL,ZLBL
C
C     Description of variables passed in common block / KNBHD /
C
C     AZMUTH    Azimuth angle of search ellipsoids major axis.
C     SAZM      Sine of azimuth angle.
C     CAZM      Cosine of azimuth angle.
C     DIP       Dip angle of search allipsoids major axis.
C     SDIP      Sine of dip angle.
C     CDIP      Cosine of dip angle.
C     RADIIA    Search ellipsoids major axis a.
C     RADIIB    Search ellipsoids minor axis b.
C     RADIIC    Search ellipsoids minor axis c.
C     AFF1      Affine ratio for axes a/b.
C     AFF2      Affine ratio for axes a/c.
C     MINHOL    Minimum number of samples in kriging nbdh.
C     MAXHOL    Maximum number of samples in kriging nbhd.
C
      COMMON / KNBHD / AZMUTH,SAZM,CAZM,DIP,SDIP,CDIP,RADIIA,
     1                 RADIIB,RADIIC,AFF1,AFF2,MINHOL,MAXHOL
      Real*4           AZMUTH,SAZM,CAZM,DIP,SDIP,CDIP,RADIIA,
     1                 RADIIB,RADIIC,AFF1,AFF2
      Integer*4        MINHOL, MAXHOL
C
C     Description of variables passed in common block / PARAM /
C
C     NVA      Number of all variables involved in cokriging.
C     NSAMP     Number of samples in cokriging system.
C     NUMHOL    Total number of sampled spatial locations.
C     IPKRIG    Type of cokriging (0 - point, 1 - block).
C     IDUAL     Dual cokriging (0 - No, 1 - Yes).
C     ILIST     Output to documentation file (0 - No, 1 - Yes).
C     IDBUG     Switch for degug output listing (0 - No, 1 - Yes).
C     ZERO      Value to indicate a zero pivot element in
C               solving the kriging system of equations.
C               If a zero pivot element is encountered, then
C               the point is not estimated.
C     Dimen     Dimension of the spatial coordinates.
      COMMON / PARAM  / NVA,NSAMP,NUMHOL,IPKRIG,IDUAL,IDBUG,ILIST,
     +                  ZERO, Dimen
      Integer*4         NVA,NSAMP,NUMHOL,IPKRIG,IDUAL,IDBUG,ILIST,
     +                  Dimen
      Real*4            ZERO
C
C     Description of variables passed in common block / FILES /
C
C     IOUT      Output file unit number.
C     INPUT     Input file unit number.
C     IPCH      Punch file unit number.
C     ITEMP     Temporary file unit number.
C
      COMMON / FILES  / IOUT,INPUT,IPCH,ITEMP
      Integer*4         IOUT,INPUT,IPCH,ITEMP
C
C
C     Description of variables passed in common block / COMCOL /
C
C     ICX       Column number of easting coordinate.
C     ICY       Column number of northing coordinate.
C     ICZ       Column number of elevation coordinate.
C     ICV(I)    Column number of variable I.
C     TEST(I)   Indicator of missing data for variable I.
C     WGT(I)    Weight to indicate estimation desired
C               for variable I.
C               If set to 0.0, then variable I is not estimated
C               during cokriging.
C
      COMMON / COMCOL / ICX,ICY,ICZ,ICV,TEST,WGT
      INTEGER*4 ICX,ICY,ICZ,ICV(MAXV)
      REAL*4    TEST(MAXV),WGT(MAXV)

C
C     Description of variables passed in common block / DRIFT /
C
C     NXCOL     Number of columns in cokriging design matrix.
C     NTERM     Number of basis functions.
C     NTERMP(*) Pointer to start of section of basis power
C               exponents for variable *.
C     EX(*)     Poly. power on X coordinate with function *.
C     EY(*)     Poly. power on Y coordinate with function *.
C     EZ(*)     Poly. power on Z coordinate with function *.
C     MVAR      Number of variables used in cokriging, not include
C               coordinates.

      COMMON / DRIFT  / NXCOL,NTERM,NTERMP,
     1                  EX,EY,EZ,MVAR,IIVAR
      REAL*4      EX(MTERM),EY(MTERM),EZ(MTERM)
      INTEGER*4   MVAR,IIVAR, NXCOL,NTERM(MAXV),NTERMP(MAXV)
C     Description of variable stored in common block / WKSP1 /.
C
C     A(*)      Storage array for variance-covariance matrix.
C
      COMMON / WKSP1  / A
      REAL*4            A(IASIZE)
C
C     Description of variables stored in common block / WKSP2 /.
C
C     X(*)      X spatial coordinate of sample *.
C     Y(*)      Y spatial coordinate of sample *.
C     Z(*)      Z spatial coordinate of sample *.
C     DIST(*)   Distance between an estimated point and
C               the sample location *.
C     IHOL(*)   Indices of the sorted samples in increasing
C               distance from an estimated point.
C     GRADE(*)  Values of attritutes stored at sampled locations
C               stored in packed vector format.
      COMMON / WKSP2  / X,Y,Z,DIST,
     1                  IHOL,GRADE
      REAL*4            X(MAXNUM),Y(MAXNUM),Z(MAXNUM),DIST(MAXNUM),
     1                  GRADE(INHOLE)
      INTEGER*4         IHOL(MAXNUM)
C
C
C     Description of variables stored in common block / WKSP3 /
C
C     NDG(*)    Pointer array to locate entries in a packed
C               symmetric array.
C     NHOL(*)   Sampled location of sample * in kriging nbhd.
C     NATR(*)   Attribute number of sample * in kriging nbhd.
C
      COMMON / WKSP3  / NDG,NHOL,NATR
      INTEGER*4         NDG(NTS),NHOL(NTS),NATR(NTS)
C
C     Description of variables stored in common block / WKSP4 /
C
C     B(*)      Storage array for cokriging design matrix.
C     BB(*)     Storage array for cokriging weights.
C     XL(*)     Storage array for solution to  [A] * [XL] = [B].
C     XR(*)     Storage array for solution to  [A] * [XR] = [Cov].
C     COV(*)    Storage array for rhs covariances.
C
      COMMON / WKSP4  / B,BB,XL,
     1                  XR,COV
      REAL*4            B(NBRHS),BB(NBRHS),XL(NBRHS),
     1                  XR(NTS),COV(NTS)
C
C     Description of variables stored in common block / WKSP5 /
C
C     EST(*)    Estimated value for attribute * at location.
C     CBKVAR(*) Block variance for attribute *.
C     COKVAR(*) Cokriging variance for attribute * at location.
C     SUM(*)    Temporary storage array.
C     G(*)      Temporary storage array.
C     GG(*)     Temporary storage array.
C     XLAG(*)   Lagrange multiplier storage array.
C     BBAR(*)   Storage array for b vector.      -1
C     DD(*)     Storage array for [X] transpose [K] [X] matrix.
      COMMON / WKSP5  / EST,CBKVAR,COKVAR,Datarec
     1                  SUM,G,GG,
     2                  XLAG,BBAR,DD
      REAL*4            EST(MAXV),CBKVAR(MAXV),COKVAR(MAXV),
     1                  SUM(MTERM),G(MTERM),GG(MTERM),DataRec(MAXV),
     2                  XLAG(MTERM),BBAR(MTERM),DD(MAXG)

C
      COMMON /FilesB/ File Prefix,IparFile,OparFile,Data File,
     +                OutFile,PchFile,Ndata
      CHARACTER*14    File Prefix*50,IparFile,OparFile,Data File,
     +                OutFile,PchFile
      LOGICAL         Intro,Point
cclong
      COMMON /logics/ Intro, Point
cc
      INTEGER*4       Ndata, Ktype, Ndiscr
      
      COMMON /Varnam B/ Units
      CHARACTER*10      Units(MAXV)

C********************************************************************
C /DataB/       -common block for input data
c Data          - data array
c NData         -Number of rows read intoData matrix
C Q1, Q2, Q3    -Data quartiles
      COMMON /DataB/ Data, Q1, Q2, Q3,
     +               Ndigits
      Real*4         Data (MAXNUM, MAXV), Q1, Q2, Q3
      Integer*4      Ndigits(MAXV)

C*******************************************************************
C Nxtic, Nytic, Nztic
c Xmin, Ymin,...
c Xtinc, Ytinc, Ztinc
      COMMON  /GRAFCOM/ Xmin, Xmax, Ymin, Ymax, Zmin, Zmax,
     1                  Xtinc, Ytinc, Ztinc, Nxtic, Nytic, Nztic
      INTEGER*4         Nxtic, Nytic, Nztic
      REAL*4            Xmin, Xmax, Ymin, Ymax, Zmin, Zmax,
     1                  Xtinc, Ytinc, Ztinc

C********************************************************************
C /MissB/
      COMMON /MissB/ Missing, count, Local,Flag
      Real*4         Missing
      Integer*4      Local, count
      Logical        Flag
