
/*
**	colorx.c
**	Sets up colors
**
*/

#include   <X11/Xlib.h>
#include   <X11/Xutil.h>

/*
**	test program external GLOBALS 
*/

extern	Display		*theDisplay;
extern	Colormap	theColormap;
extern	int		theDepth;	/* -- How many color planes, if any? */
extern	unsigned long	theBlackPixel;	/* -- System drawing color           */
extern	unsigned long	theWhitePixel;	/* -- System background color        */

#define			maxPixels	66

unsigned long		thePixels[ maxPixels ];

/*
**    Set up English text for colors
**
*/
char			*theColorNames[ maxPixels ] =
	 		{	"aquamarine",		/* -- 0   */
		 		"black",		/* -- 1   */
				"blue",			/* -- 2   */
				"BlueViolet",		/* -- 3   */
				"brown",		/* -- 4   */
				"CadetBlue",		/* -- 5   */
				"coral",		/* -- 6   */
				"CornflowerBlue",	/* -- 7   */
				"cyan",			/* -- 8   */	
				"DarkGreen",		/* -- 9   */
				"DarkOliveGreen",	/* -- 10  */
				"DarkOrchid",		/* -- 11  */
				"DarkSlateBlue",	/* -- 12  */
				"DarkSlateGrey",	/* -- 13  */
				"DarkTurquoise",	/* -- 14  */
				"DimGrey",		/* -- 15  */
				"firebrick",		/* -- 16  */
				"ForestGreen",		/* -- 17  */
				"gold",			/* -- 18  */
				"goldenrod",		/* -- 19  */
				"grey",			/* -- 20  */
				"green",		/* -- 21  */
				"GreenYellow",		/* -- 22  */
				"IndianRed",		/* -- 23  */
				"Khaki",		/* -- 24  */
				"LightBlue",		/* -- 25  */
				"LightGrey",		/* -- 26  */
				"LightSteelBlue",	/* -- 27  */
				"LimeGreen",		/* -- 28  */
				"magenta",		/* -- 29  */
				"maroon",		/* -- 30  */
				"MediumAquamarine",	/* -- 31  */
				"MediumBlue",		/* -- 32  */
				"MediumForestGreen",	/* -- 33  */
				"MediumGoldenrod",	/* -- 34  */
				"MediumOrchid",		/* -- 35  */
				"MediumSeaGreen",	/* -- 36  */
				"MediumSlateBlue",	/* -- 37  */
				"MediumSpringGreen",	/* -- 38  */
				"MediumTurquiose",	/* -- 39  */
				"MediumVioletRed",	/* -- 40  */
				"MidnightBlue",		/* -- 41  */
				"Navy",			/* -- 42  */
				"orange",		/* -- 43  */
				"OrangeRed",		/* -- 44  */
				"orchid",		/* -- 45  */
				"PaleGreen",		/* -- 46  */
				"Pink",			/* -- 47  */
				"Plum",			/* -- 48  */
				"red",			/* -- 49  */	
				"salmon",		/* -- 50  */
				"SeaGreen",		/* -- 51  */
				"Sienna",		/* -- 52  */
				"SkyBlue",		/* -- 53  */
				"SlateBlue",		/* -- 54  */
				"SpringGreen",		/* -- 55  */
				"SteelBlue",		/* -- 56  */
				"tan",			/* -- 57  */
				"thistle",		/* -- 58  */
				"turquiose",		/* -- 59  */
				"violet",		/* -- 60  */
				"violetred",		/* -- 61  */
				"wheat",		/* -- 62  */
				"white",		/* -- 63  */
				"yellow",		/* -- 64  */
				"YellowGreen"		/* -- 65  */
		};



/*
**	setColorWithName sets the given graphics context GC with the foreground
**	color named by the English text name.
**
**	It is not very efficient.  Ideally, all colors would be
**	allocated at the beginning of a program.
**
*/

setColorWithName( theGC, theName )
     
     GC	theGC;
     char	theName[];
     
{	/* -- function setColorWithName */
  int		i;
  
  i = 0;
  if ( theDepth > 1 )
    {
      while( ( strcmp( theName, theColorNames[ i ] ) != 0 ) &&
	    ( i < maxPixels  ) )
	{
	  i++;
	  if (i == maxPixels) break;
	}
      
      if ( i < maxPixels )
	{
	  /* -- Set the foreground color */
	  XSetForeground( theDisplay,
			 theGC,
			 thePixels[ i ] );
	  
	}
      else
	{
	  /* -- Set the foreground color to white*/
	  printf("Your color was not found: defaulting to white...\n");
	  strcpy(theName, "white");
	  i = 63;
	  XSetForeground( theDisplay,
			 theGC,
			 thePixels[ i ] );
	}
    }
  else
    {
      if ( strcmp( theName, "black" ) == 0 )
	{
	  strcpy(theName, "black");
	  i = 1;
	  XSetForeground( theDisplay,
			 theGC,
			 thePixels[ i ] );
	}
      else
	{ 
	  strcpy(theName, "white");
	  i = 63;
	  XSetForeground( theDisplay,
			 theGC,
			 thePixels[ i ] );
	}
    }  
}	/* -- function setColorWithName */


/*
**	initDefaultColors Attempts to set up a local color table with 
**	the default X11 colors.  After this, colors can be accessed
**	by a color number, rather than an English name.
**	Note that this function tries to use the default
**	color table.  On a monochrome system, all colors
**	will turn into BlackPixel.  The background white
**	color will turn into WhitePixel.
**
*/


initDefaultColors()


{	/* -- function initDefaultColors */
	unsigned long	theBlackPixel, theWhitePixel;
	XColor		theRGBColor, theHardwareColor;
	int		theStatus;
	int		i;
	

	if ( theDepth > 1 )
	  {
	    for( i = 0; i < maxPixels; i++ )
	      {
		theStatus = XLookupColor( theDisplay,
					 theColormap,
					 theColorNames[ i ],	
					 &theRGBColor,
					 &theHardwareColor );
		
		if ( theStatus != 0 )
		  {
		    theStatus = XAllocColor( theDisplay, 
					    theColormap,
					    &theHardwareColor );
		    
		    if ( theStatus != 0 )
		      {
			thePixels[ i ] = theHardwareColor.pixel;
		      }
		    else
		      {	
			thePixels[ i ] =  theBlackPixel;
		      }
		  }
	      }
	  }
	else	
	  {
	    /*
	     **	Monochrome system
	     */
	    
	    for( i = 0; i < maxPixels; i++ )
	      {
		if ( strcmp( "black", theColorNames[ i ] ) == 0 )
		  {
		    thePixels[ i ] = theBlackPixel;
		  }
		else
		  {
		    thePixels[ i ] = theWhitePixel;
		  }
	      }
	  }
	
      }	/* -- function initDefaultColors */


/*
**	setColor sets the GC theGC to have a foreground color of colorNumber.
**
*/

setColor( theGC, colorNumber )
     
     GC	theGC;
     int	colorNumber;
     
{	/* -- function setColor */
  
  if ( theDepth > 1 )
    {
      if ( ( colorNumber < maxPixels ) && ( colorNumber >= 0 ) )
	{
      /* -- Set the foreground color */
	  XSetForeground( theDisplay,
			 theGC,
			 thePixels[ colorNumber ] );
	}
    }
  else
    {
      if ( colorNumber != 1 )  colorNumber = 63; 
      XSetForeground( theDisplay,
		     theGC,
		     thePixels[ colorNumber ] );
    }      
}	/* -- function setColor */

/* _______________________________________________________________________ */


/*
 **	end of file colorx.c
 **
 */



