
/*
**	fontx.c
**	Sets up fonts
**
*/

#include   <X11/Xlib.h>
#include   <X11/Xutil.h>

/*
**	test program external GLOBALS 
*/

extern	Display		*theDisplay;

#define			maxPixels	66
unsigned long		thePixels[ maxPixels ];

/*
**    Set up English text for fonts
**
*/
char			*theFontNames[ maxPixels ] =
{	"aquamarine",		/* -- 0   */
		 		"black",		/* -- 1   */
				"blue",			/* -- 2   */
				"BlueViolet",		/* -- 3   */
				"brown",		/* -- 4   */
				"CadetBlue",		/* -- 5   */
				"coral",		/* -- 6   */
				"CornflowerBlue",	/* -- 7   */
				"cyan",			/* -- 8   */	
				"DarkGreen",		/* -- 9   */
				"DarkOliveGreen",	/* -- 10  */
				"DarkOrchid",		/* -- 11  */
				"DarkSlateBlue",	/* -- 12  */
				"DarkSlateGrey",	/* -- 13  */
				"DarkTurquoise",	/* -- 14  */
				"DimGrey",		/* -- 15  */
				"firebrick",		/* -- 16  */
				"ForestGreen",		/* -- 17  */
				"gold",			/* -- 18  */
				"goldenrod",		/* -- 19  */
				"grey",			/* -- 20  */
				"green",		/* -- 21  */
				"GreenYellow",		/* -- 22  */
				"IndianRed",		/* -- 23  */
				"Khaki",		/* -- 24  */
				"LightBlue",		/* -- 25  */
				"LightGrey",		/* -- 26  */
				"LightSteelBlue",	/* -- 27  */
				"LimeGreen",		/* -- 28  */
				"magenta",		/* -- 29  */
				"maroon",		/* -- 30  */
				"MediumAquamarine",	/* -- 31  */
				"MediumBlue",		/* -- 32  */
				"MediumForestGreen",	/* -- 33  */
				"MediumGoldenrod",	/* -- 34  */
				"MediumOrchid",		/* -- 35  */
				"MediumSeaGreen",	/* -- 36  */
				"MediumSlateBlue",	/* -- 37  */
				"MediumSpringGreen",	/* -- 38  */
				"MediumTurquiose",	/* -- 39  */
				"MediumVioletRed",	/* -- 40  */
				"MidnightBlue",		/* -- 41  */
				"Navy",			/* -- 42  */
				"orange",		/* -- 43  */
				"OrangeRed",		/* -- 44  */
				"orchid",		/* -- 45  */
				"PaleGreen",		/* -- 46  */
				"Pink",			/* -- 47  */
				"Plum",			/* -- 48  */
				"red",			/* -- 49  */	
				"salmon",		/* -- 50  */
				"SeaGreen",		/* -- 51  */
				"Sienna",		/* -- 52  */
				"SkyBlue",		/* -- 53  */
				"SlateBlue",		/* -- 54  */
				"SpringGreen",		/* -- 55  */
				"SteelBlue",		/* -- 56  */
				"tan",			/* -- 57  */
				"thistle",		/* -- 58  */
				"turquiose",		/* -- 59  */
				"violet",		/* -- 60  */
				"violetred",		/* -- 61  */
				"wheat",		/* -- 62  */
				"white",		/* -- 63  */
				"yellow",		/* -- 64  */
				"YellowGreen"		/* -- 65  */
		};



/*
**	setFontWithName sets the given graphics context GC with the foreground
**	font named by the English text name.
**
**	It is not very efficient.  Ideally, all fonts would be
**	allocated at the beginning of a program.
**
*/

setFontWithName( theGC, theName )
     
     GC	theGC;
     char	theName[];
     
{	/* -- function setFontWithName */
  int		i;
  
  i = 0;
  while( ( strcmp( theName, theFontNames[ i ] ) != 0 ) &&
	( i < maxPixels  ) )
    {
      i++;
      if (i == maxPixels) break;
    }
  
  if ( i < maxPixels )
    {
      /* -- Set the foreground font */
      XSetForeground( theDisplay,
		     theGC,
		     thePixels[ i ] );
      
    }
  else
    {
      /* -- Set the foreground font to white*/
      printf("Your font was not found: defaulting to white...\n");
      strcpy(theName, "white");
      i = 63;
      XSetForeground( theDisplay,
		     theGC,
		     thePixels[ i ] );
    }
}
}	/* -- function setFontWithName */


/*
**	initDefaultFonts Attempts to set up a local font table with 
**	the default X11 fonts.  After this, fonts can be accessed
**	by a font number, rather than an English name.
**	Note that this function tries to use the default
**	font table.  On a monochrome system, all fonts
**	will turn into BlackPixel.  The background white
**	font will turn into WhitePixel.
**
*/


initDefaultFonts()
{	/* -- function initDefaultFonts */
  unsigned long	theBlackPixel, theWhitePixel;
  XFont		theRGBFont, theHardwareFont;
  int		theStatus;
  int		i;
  
  
  for( i = 0; i < maxPixels; i++ )
    {
      theStatus = XLookupFont( theDisplay,
			      theFontmap,
			      theFontNames[ i ],	
			      &theRGBFont,
			      &theHardwareFont );
      
      if ( theStatus != 0 )
	{
	  theStatus = XAllocFont( theDisplay, 
				 theFontmap,
				 &theHardwareFont );
	  
	  if ( theStatus != 0 )
	    {
	      thePixels[ i ] = theHardwareFont.pixel;
	    }
	  else
	    {	
	      thePixels[ i ] =  theBlackPixel;
	    }
	}
    }
}


setFont( theGC, fontNumber )
     
     GC	theGC;
     int	fontNumber;
     
{	/* -- function setFont */
  
  if ( ( fontNumber < maxPixels ) && ( fontNumber >= 0 ) )
    {
      /* -- Set the foreground font */
      XSetForeground( theDisplay,
		     theGC,
		     thePixels[ fontNumber ] );
    }
}

/* _______________________________________________________________________ */


/*
 **	end of file fontx.c
 **
 */



