C*********************************************************************
C*** File Name: \GEOEAS\CORRES\CORRES.FOR 
C*** Source code for the program CORRES
C*********************************************************************
C                            PROGRAM CORRES
C
C This program is a correspondences analysis program.  Correspondence
C analysis is a technique for displaying the rows and columns of a 
C data matrix (primarily, a two-way contingency table) as points in 
C dual low-dimensional vector spaces.  Options allow selection of a 
C subset variables from a Geo-EAS data file, and the number of factors
C to consider in the correspondence analysis.  Corres produces a
C variety of outputs, and allows flexibility in the choice of outputs.
C In order to produce output compatible with Geo-EAS programs, these
C outputs have been isolated into four separate Geo-EAS format result
C files.  The contents of each of these files may be viewed within the
C program, using a "Browse" screen which allows the user to scroll the 
C contents fo the file on the screen.  An option allows a subset of any 
C one of the four output files to be merged with data from an existing
C Geo-EAS file to produce a "custom" output file.
C
C The main program performs the following:
C   Reads in the system defaults
C   Sets up the screen
C   Calls Main Menu to display the main screen and menu and
C     to process the user's requests
C   Writes to the system defaults file
C
 
      PROGRAM CORRES
      
      INCLUDE 'corres.inc'
      INCLUDE 'screen.str'
      character*80 geoeas_home
      
      call get_geoeas_dir(10,geoeas_home)
      CALL Read Structure  (geoeas_home(1:lenstr(geoeas_home))
     &     //'/corres/corres.scr')

      CALL set up screen()
cclongadded this as in test.for

      CALL Clear Screen
      CALL Read System Defaults
      IF (Intro) CALL Intro Screen
      CALL Main Menu
      CALL Write System Defaults (FilePrefix, DataFile)
      END

C********************************************************************
C SUBROUTINE Get FGroup 
C
C This routine gets the value for a group of screen fields.
C
  	SUBROUTINE Get FGroup 
	
        IMPLICIT INTEGER*4 (A-Z)

	PARAMETER (Main Menu Field = 3)
	PARAMETER (Menu Field      = 4)

		INCLUDE 'screen.str'
	
 10	CALL Get Field	
	IF (SC Fld Type (Next Field) .NE. Main Menu Field .AND.
     +      SC Fld Type (Next Field) .NE. Menu Field          ) 
     +     GOTO 10

	RETURN
	END

C*******************************************************************
C SUBROUTINE Intro Screen 
C
C  This routine displays the screen frame/labels/fields for the 
C  introductory screen.           
C
	SUBROUTINE Intro Screen

	CALL Clear Screen
	CALL Set Screen (1)
	CALL Display Frame
	CALL Display Labels
cclong	CALL Locate Cursor (25, 0)
	CALL Locate Cursor (23, 0)
cc
	CALL Flash (20, 2, 'Press any key to begin the program ')
	RETURN
        END  

C*********************************************************************
C SUBROUTINE Display Flds 
C
C This routine displays the current screen fields.
C
        SUBROUTINE Display Flds

        IMPLICIT INTEGER*4 (A-Z)

		INCLUDE 'screen.str'

        INTEGER*4   Main Menu Field, Menu Field
  
        PARAMETER (Main Menu Field = 3)
        PARAMETER (Menu Field      = 4)


ccc        DO 10 i = BG Screen (Curt Screen),
ccc     +            End Screen (Curt Screen)
        DO 10 i = BG Screen (Current Screen),
     +            End Screen (Current Screen)
           IF (SC Fld Type (i) .NE. Main Menu Field .AND.
     +         SC Fld Type (i) .NE. Menu Field            )
     +        CALL Write Field (i, SC  Attribute (i))
10      CONTINUE

        RETURN
        END           

C************************************************************************
C SUBROUTINE Read System Defaults 
C 
C This routine reads the system defaults file 'geoeas.def' and  
C initializes the system defaults.
C
        SUBROUTINE Read System Defaults
        
        IMPLICIT INTEGER*4 (A-Z)

		INCLUDE  'corres.inc'

        CHARACTER   String*80

        OPEN (UNIT=10,FILE ='geoeas.def',STATUS = 'UNKNOWN',
     +        ERR= 999)

        READ (10, '(A)', ERR = 999,END= 999) FilePrefix
        READ (10, '(A)', ERR = 999,END= 999) DataFile
        READ (10, '(A)', ERR = 999,END= 999) String

C *** Intro flag embedded as INTRO=ON, INTRO=anything else
C *** Intro is a global logical
        Intro = (String (7:8) .EQ. 'ON' .OR.
     +           String (7:8) .EQ. 'on')

        CALL Set Character Value ('PREFIX    ',FilePrefix)
        CALL Set Character Value ('DATAFILE  ',DataFile)

        CLOSE (10)
        RETURN
         
 999    CLOSE (10)
        FilePrefix = ' '
        Intro = .TRUE.

        RETURN
        END

C************************************************************************
C SUBROUTINE Write System Defaults (Prefix, File)
C
C Formal Arguments:
C  Prefix   -  File prefix 
C  File     -  Data file name
C
C This routine writes the file prefix and the data file name to the
C system defaults file 'geoeas.def'.  
C

         SUBROUTINE Write System Defaults (Prefix, File)
        
        IMPLICIT INTEGER*4 (A-Z)

		INCLUDE  'corres.inc'

        CHARACTER*50 Prefix
        CHARACTER*14 File

        OPEN (UNIT = 10, FILE = 'geoeas.def', STATUS = 'UNKNOWN',
     +        ERR= 999)

        WRITE (10, '(A)', ERR = 999) Prefix
        WRITE (10, '(A)', ERR = 999) File

        IF (Intro) THEN
           WRITE (10, '(A)', ERR = 999) 'INTRO=ON'
        ELSE
           WRITE (10, '(A)', ERR = 999) 'INTRO=OFF'
        ENDIF


 999    CLOSE (10)

        RETURN
        END


C**********************************************************************   
C SUBROUTINE Get New Screen (Screen Number) 
C
C Formal Arguments:
C  Screen Number  - Stores the screen number.  
C
C This routine sets the current screen to screen number and then displays
C the frame and labels for that screen. 
C

        SUBROUTINE Get New Screen (Screen Number)
         
        IMPLICIT INTEGER*4 (A-Z)
       
        INTEGER*4   Screen Number        

        CALL Clear Screen
        CALL Set Screen (Screen Number)
        CALL Display Frame
        CALL Display Labels

        RETURN
        END

