cc$LARGE
C Polygon.for
C----------------------------------------------------------------------
C Read Polygon File - Read and process polygon file for program KRIGE
C
	SUBROUTINE Read Polygon File (File Name, Error)
	IMPLICIT INTEGER*4 (A-Z)

	PARAMETER (MAXPOLY = 100)

        INCLUDE 'krige.dec'
	
	LOGICAL		Inside, Inclusive, Error
	INTEGER*4	IPoly, Ivertex, Npoly, Nvertex, Col, Row
	INTEGER*4	PtLoc, Loc
	REAL*4		PolygonX (MAXPOLY), PolygonY (MAXPOLY), X0, Y0
	CHARACTER*65	File Name

C open polygon file
	CALL PutStr ('Processing polygon file '//PolyFile, 38,20,2,2)
	OPEN (UNIT = 10, FILE = File Name, STATUS = 'OLD', ERR = 999)

C read number of polygons record
	READ (10, *, ERR = 999) Npoly

C process each Polygon ...
	DO 10 IPoly = 1, Npoly
	   READ (10, *, ERR = 999) Inc, Nvertex
	   Inclusive = (Inc .EQ. 0)

C read all vertices into Polygon arrays
	   DO 20  Ivertex = 1, Nvertex
		READ (10, *, ERR = 999) PolygonX (Ivertex),
     +                                  PolygonY (Ivertex)
 20	   CONTINUE

C check each grid node against polygon boundry
	   DO 30 Col 	= 1, NxCells
              X0   = Xorigin + (Col-1) * XcellSize 
	      DO 30 Row = 1, NyCells
                 Y0 = Yorigin + (Row-1) * YcellSize
	         Loc = PtLoc (Nvertex, PolygonX, PolygonY, X0, Y0)
	         Inside = (Loc .GE. 0)
   	         
C determine if grid node is to be excluded
		 IF (Inside) THEN
                    IF (.NOT. Inclusive) Excluded (Col, Row) = .TRUE.
	         ELSE
	            IF (Inclusive) Excluded (Col, Row) = .TRUE.
	         ENDIF
 30	   CONTINUE	  
 10	CONTINUE
	CLOSE (10)
	Error = .FALSE.
	CALL ERASE (20, 2, 76)
	RETURN

C error opening or reading file ...
 999	CLOSE (10)
	CALL ERASE (20, 2, 76)
	Error = .TRUE.
	RETURN
	END

C---------------------------------------------------------------------
C Initgrid - initialize kriging grid
C
 	SUBROUTINE InitGrid
	IMPLICIT INTEGER*4 (A-Z)

        INCLUDE 'krige.dec'

C Change Excluded array to .FALSE. (every node included)
	CALL PutStr ('Initializing Grid   ', 20, 20, 2, 2)
        DO 35 I = 1, MAXXGRID
           DO 35 J = 1, MAXYGRID
              Excluded (I, J) = .FALSE.
 35     CONTINUE
	CALL ERASE (20, 2, 76)
	RETURN
	END


C-----------------------------------------------------------------
      INTEGER FUNCTION PTLOC(NP,X,Y,XP,YP)
C*****************************************************************
C***    THIS SUBPROGRAM DETERMINES WHETHER A POINT LIES WITHIN A
C***    CLOSED BOUNDARY.
C***    IF PTLOC =-1  THE POINT IS OUTSIDE.
C***               0  THE POINT LIES ON THE BOUNDARY.
C***               1  THE POINT LIES WITHIN THE BOUNDARY.
C***               2  THE POINT LIES ON A NODE
C***    PROGRAM PTLOC WAS WRITTEN BY JOHN K. HALL OF THE
C***    GEOLOGICAL SURVEY OF ISRAEL. DOCUMENTATION AND
C***    EXPLANATION OF THIS ALGORITHM WERE PUBLISHED IN
C***    MATHEMATICAL GEOLOGY, VOL 7,NO. 1, FEB., 1975
C***    PROGRAM SLIGHTLY MODIFIED BY W.D. GRUNDY,USGS,1979, JUNE 1985
C*****************************************************************
      DIMENSION X(1),Y(1)
      DOUBLE PRECISION C180
      DATA C180/.9999999998D0/
      IEC=0
      TSC=0.0
      SC=0.0
      SC1=0.0
      TCC=1.0
      CC=1.0
      X1=X(NP)-XP
      Y1=Y(NP)-YP
      R1=SQRT(X1*X1+Y1*Y1)
      IF(R1) 10,150,10
   10 DO 110 I=1,NP
      X2=X(I)-XP
      Y2=Y(I)-YP
      R2=SQRT(X2*X2+Y2*Y2)
      VPL=R1*R2
      IF(R2) 20,150,20
   20 ST=(X1*Y2-X2*Y1)/VPL
      CT=(X1*X2+Y1*Y2)/VPL
      IF(CT+C180) 130,130,30
   30 IF(I-NP) 50,40,40
   40 IF(MOD(IEC,2)) 120,120,140
   50 TSC=SC*CT+CC*ST
      TCC=CC*CT-SC*ST
      IF(TSC*SC) 60,60,90
   60 IF(TSC*SC1) 70,70,90
   70 IF(TSC*ST) 80,90,90
   80 IEC=IEC+1
   90 SC=TSC
      CC=TCC
      X1=X2
      Y1=Y2
      R1=R2
      IF(SC) 100,110,100
  100 SC1=SC
  110 CONTINUE
  120 PTLOC=-1
      GO TO 170
  130 PTLOC=0
      GO TO 170
  140 PTLOC=1
      GO TO 170
  150 PTLOC=2
  170 RETURN
      END





