cc$LARGE
C
C Process.for
C----------------------------------------------------------------------
C Perform Process - perform application-specific functions for KRIGE
C
	SUBROUTINE Perform Process (Field Name, Error, Repaint)
	IMPLICIT INTEGER*4 (A-Z)
	PARAMETER (Alpha Field     = 0)
	PARAMETER (Integer Field   = 1)
	PARAMETER (Real Field      = 2)
	PARAMETER (Toggle Field    = 5)

        INCLUDE 'krige.dec'
        INCLUDE '../common/screen.str'

	COMMON		/GRVAR/ OldXvar, OldYvar

	CHARACTER	Message*75, OldFile*14, FileName*65
	CHARACTER*10	Field Name,  MainMenu, Dummy, Return Field
	CHARACTER*13	Scratch
	INTEGER*4	Fld Index, Index of Field, Field Type
	INTEGER*4	LENSTR, Position, I, OldXvar, OldYvar
	INTEGER*4	Field1, Field2, Field3, Pos
	LOGICAL		Error, Repaint, PutMessage
	LOGICAL		Started, Fexist
        LOGICAL       GetYesNo, OverWrite
	REAL*4		Temp

	DATA MainMenu   /'MAINMENU  '/
	DATA Started	/.FALSE./

C initialize
	Error		= .FALSE.
	Repaint		= .FALSE.	
	PutMessage	= .FALSE.
	OverWrite 	= .TRUE.
	FExist		= .FALSE.
	Index of Field	= Fld Index (Field Name)
	Return Field	= SC Field Name (Next Field)
	Field Type	= SC Field Type (Index of Field)
	Pos		= 0
	Message		= ' '

C Get a real value, or toggle position
	IF (Field Type .EQ. Real Field) THEN
	   CALL Get 4 Floating Point Value (Field Name, Temp)
	ELSE IF (Field Type .EQ. Toggle Field) THEN
           CALL Get Toggle Value (Field Name, Position, Dummy)
	ELSE IF (Field Type .EQ. Alpha Field) THEN
	   CALL Get Character Value (Field Name, Message)
	   CALL Left Justify (Message)
	   CALL Set Character Value (Field Name, Message)
	ENDIF

C----------------------------------------------------------------------
C get file prefix
	IF     (Field Name .EQ. 'FILEPREFIX') THEN
	  CALL Get Character Value (Field Name, File Prefix)

C----------------------------------------------------------------------
C get parameter file name, write parameters
	ELSEIF (Field Name .EQ. 'OPARFILE  ') THEN
	  CALL Get Character Value (Field Name, OParFile)
	  CALL Get File Name (File Name, File Prefix, OParFile, Fexist)
	  IF (Fexist) OverWrite	=
     +	     GetYesNo (20, 2,'OverWrite existing file? <Y/N> ')
	  IF (OverWrite) THEN
	     CALL Save Parameters (File Name, Error)
	     IF (Error) THEN
	        CALL Blank File (Field Name, OparFile)
	        Message = 'Parameters NOT saved... (press any key)'
	     ELSE
	        CALL Flash (20, 2, 'Parameters saved... (press a key)')
	     ENDIF
	  ELSE
	     CALL Blank File (Field Name, OparFile)
	  ENDIF

C----------------------------------------------------------------------
C get parameter file name, read parameters
	ELSEIF (Field Name .EQ. 'IPARFILE  ') THEN
	  CALL Get Character Value (Field Name, IParFile)
	  CALL Get File Name (File Name, File Prefix, IParFile, Fexist)
	  IF (Fexist) THEN
             CALL Read Parameters (File Name, Error)
             IF (Error) THEN
                CALL ReInitialize
                CALL Display Fields
                Message = 'ERROR in parameter file... (press any key)'
             ELSE
    	       CALL Set Character Value ('FILEPREFIX', File Prefix)
	       CALL Display One Field   ('FILEPREFIX')
	       CALL Get FileName (File Name, File Prefix,
     +			 Data File, Fexist)
cclong 94/3/1
ccc               CALL Set Character Value ('DATAFILE  ', Data File)
cclong 94/3/1
	       IF (.NOT. Fexist) THEN
	          Error = .TRUE.
	          Message =
     +		 'ERROR - Data file '//Data File//' not found'
	          CALL Blank File (Field Name, Data File)
	          CALL Reinitialize
	       ELSE
	          CALL Read Header Data (File Name, Error)
	          IF (Error) THEN
	            Message = 
     +	 	   'ERROR reading header data in '//Data File 
	           CALL Blank File (Field Name, Data File)
                   CALL ReInitialize
	          ELSE
	           CALL Read Coordinate Values (Error)
	           IF (Error) THEN
	             Message = 
     +	 	    'ERROR reading coordinate data in '//Data File 
	              CALL Blank File (Field Name, Data File)
	              CALL Reinitialize
	 	   ELSE
	              CALL Assign Screen Fields
	           CALL Flash (20,2,'Parameters loaded (press any key)')
	           ENDIF
	          ENDIF
	       ENDIF
	    ENDIF
	  ELSE
	    CALL Flash (20, 2,
     +	    'Parameter file not found... (press a key)')
	    CALL Blank File (Field Name, IparFile)
	  ENDIF

C----------------------------------------------------------------------
C get data file name, read header data
	ELSEIF (Field Name .EQ. 'DATAFILE  ') THEN
	  IF (Started) Old File = Data File
	  CALL Get Character Value (Field Name, Data File)
	  CALL Get FileName (File Name, File Prefix, Data File, Fexist)
	  IF (Old File .EQ. Data File .AND.
     +		 Data File (1:1) .EQ. ' ') THEN
	        Return Field = 'OPTIONS   '
		GOTO 80
	  ENDIF
	  IF (.NOT. Fexist) THEN
	     Error = .TRUE.
	     Message = 'ERROR - '//Data file//' not found'
             Return Field = 'OPTIONS   '
	     CALL Blank File (Field Name, Data File)
	     CALL Reinitialize
	     CALL Display Fields
	  ELSE
	    CALL Read Header Data (File Name, Error)
	    IF (Error) THEN
	      CALL Flash (20, 2,
     +	 	'ERROR reading header data in '//Data File) 
	      Error = .FALSE. 
	      CALL Blank File (Field Name, Data File)
              CALL ReInitialize
	      CALL Display Fields
              Return Field = 'OPTIONS   '
	    ELSE

C Set up defaults for the run
	      Started = .TRUE.
	      CALL Display One Field  ('GR XVAR   ')   
	      CALL Display One Field  ('GR YVAR   ')
	      K = Index (Data File, '.')
	      IF (K .NE. 0) THEN
	         K = AMIN0 (K, 9)
	      ELSE
	         L = LENSTR (Data File)
	         K = AMIN0 (L, 9)
	      ENDIF
	      OutFile = Data File (1:K-1)//'.grd'
	      CALL Set Character Value ('OUTFILE   ', OutFile)
	      IparFile = Data File (1:K-1)//'.kpf'
	      OparFile = IparFile
              CALL Set Character Value ('IPARFILE  ', IparFile)
	      CALL Set Character Value ('OPARFILE  ', OparFile)
	      CALL Display One Field ('OUTFILE   ')
	      Title = OutFile//' - kriged estimates of data from '
     +		 //Data File
	      CALL Set Character Value ('OUTTITLE  ', Title)
	      CALL Display One Field   ('OUTTITLE  ')
            ENDIF
	  ENDIF

C----------------------------------------------------------------------
C get the polygon file name
	ELSEIF (Field Name .EQ. 'POLYFILE  ') THEN
	  CALL Get Character Value (Field Name, PolyFile)
	  CALL Get File Name (File Name, File Prefix, PolyFile, Fexist)
	  IF (.NOT. Fexist .OR. PolyFile (1:1) .EQ. ' ') THEN
	     CALL Flash (20, 2, 'File not found... (press a key)')
	     CALL Blank File (Field Name, Poly File)
	  ENDIF

C----------------------------------------------------------------------
C get the estimates file name
	ELSEIF (Field Name .EQ. 'OUTFILE   ') THEN
	  CALL Get Character Value (Field Name, OutFile)
	  CALL Left Justify (OutFile)
	  CALL Get File Name (File Name, File Prefix, OutFile, Fexist)
	  IF (Fexist) OverWrite	=
     +	     GetYesNo (20, 2,'OverWrite existing file? <Y/N> ')
	  IF (.NOT. OverWrite .OR. OutFile (1:1) .EQ. ' ') THEN
	     CALL Flash (20, 2, 'Press any key to try again')
	     Return Field = Field Name
	  ENDIF

C----------------------------------------------------------------------
C get Kriging type
	ELSEIF (Field Name .EQ. 'TY KRIGE  ') THEN
	  Ktype = Position
	  IF (Ktype .EQ. ORDINARY) THEN
	    DO 33 I = 1, MAXVARK 
	       Mglobal (I) = 0.0
 33	    CONTINUE
	    CALL Set Character Value ('VM MGLOBAL', '             ')
	  ENDIF 

C----------------------------------------------------------------------
C Point or block kriging, number of discretisation points
	ELSEIF (Field Name .EQ. 'TY PTBLOCK') THEN
	  Point = (Position .EQ. 0)
	  Ndiscr = Position + 1

C----------------------------------------------------------------------
C get the search options
	ELSEIF (Field Name (1:2) .EQ. 'SE') THEN
          Field1 = Fld Index ('SE MAXPO  ')
          Field2 = Fld Index ('SE MINPT  ')
          Field3 = Fld Index ('SE MAXEMPT')
	  IF      (Field Name (3:10) .EQ. ' RMAJOR ') THEN
	    IF (Temp .LE. 0.0) THEN
	       Error 	= .TRUE.
	       Message	= 'Radius must be > 0 - (press any key)'
	       Temp 	= SrMajor
	       Return Field = Field Name
	    ELSE
	       SrMajor = Temp 
	       IF (Character .EQ. 13) THEN
	         Dummy = 'SE RMINOR '
	         CALL Get Character Value (Dummy, Scratch)
	         IF (Scratch .EQ. '             ' .OR. 
     +	            SrMinor .GT. SrMajor) THEN
	            SrMinor = SrMajor
	            CALL Set 4Floating Point Value (Dummy, SrMinor, 3)
	         ENDIF 
	       ENDIF 
	    ENDIF
	  ELSE IF (Field Name (3:10) .EQ. ' RMINOR ') THEN
	    IF (Temp .LE. 0.0) THEN
	       Error 	= .TRUE.
	       Message	= 'Radius must be > 0 - (press any key)'
	       Temp 	= SrMinor
	       Return Field = Field Name
	    ELSE IF (Temp .GT. SrMajor) THEN
	       Error 	= .TRUE.
	       Message 	=
     +	 'Minor radius may not be greater than major - (press any key)'
	       Temp 	= SrMinor
	       Return Field = Field Name
	    ELSE
	       SrMinor 	= Temp 
	    ENDIF
      	  ELSE IF (Field Name (3:10) .EQ. ' ANGLE  ') THEN
	    IF (Temp .LT. 0.0 .OR. Temp .GE. 180.0) THEN
	       Error 	= .TRUE.
	       Message	=
     +  'Angle must be .GE. 0  and .LT. 180 - (press any key)'
	       Temp 	= 0.0
	       SrAngle 	= 0.0
	       Return Field = Field Name
	    ELSE
	       SrAngle = Temp
	    ENDIF
	  ELSE IF (Field Name (3:10) .EQ. ' DTYPE  ') THEN
	     Euclidean = (Position .EQ. 0)
	  ELSE IF (Field Name (3:10) .EQ. ' ETYPE  ') THEN
             IF (Position .EQ. 0) THEN
               Nsector = 1      
               SC Members  (Field1) = 24
               SC Members  (Field3) = 1
               SC Position (Field3) = 0
               CALL Write Field (Field3, SC Attribute (Field3))
             ELSEIF (Position .EQ. 1) THEN
               Nsector = 4
               SC Members  (Field1) = 16
               SC Members  (Field3) = 4
             ELSEIF (Position .EQ. 2) THEN
               Nsector = 8
               SC Members  (Field1) = 8
               SC Members  (Field3) = 8
             ENDIF 
	  ELSE IF (Field Name (3:10) .EQ. ' MAXPO  ') THEN
            MaxPs = Position + 1
            MaxP  = MaxPs * Nsector
	    IF (Maxp .GT. 24) THEN
               SC Members  (Field2) = 24
	    ELSE
               SC Members  (Field2) = MaxP-1
	    ENDIF
	  ELSE IF (Field Name (3:10) .EQ. ' MINPT  ') THEN
	    MinP = Position + 1
	  ELSE IF (Field Name (3:10) .EQ. ' MAXEMPT') THEN
	    MaxEmpty = Position + 1
	  ENDIF

C----------------------------------------------------------------------
C get the grid options
	ELSEIF (Field Name (1:2) .EQ. 'GR') THEN
	   IF (Field Name .EQ. 'GR XVAR   ') THEN
	      OldXvar = Xvar
              Xvar = Position + 1
	   ELSE IF (Field Name .EQ. 'GR YVAR   ') THEN
	      OldYvar = Yvar
              Yvar = Position + 1
	      IF (Xvar .EQ. Yvar) THEN
                 Error   = .TRUE.
	         Message = 'ERROR - X and Y must be different variables '
                 Return Field = 'GR XVAR   '
	      ELSE
	        IF (OldXvar .NE. Xvar .AND. OldYvar .NE. Yvar .OR.
     +		     Ndata .EQ. 0) THEN
	         OldXvar = Xvar
	         OldYvar = Yvar
	         CALL Read Coordinate Values (Error)
	         IF (Error) THEN
	           CALL Beep
	           Message = 
     +		  'ERROR reading coordinate values from '//Data File 
	           CALL Blank File (Field Name, Data File)
                   CALL ReInitialize
	           CALL Display Fields
                   Return Field = 'OPTIONS   '
	         ELSE
	           CALL Display Fields
	         ENDIF
	        ENDIF
	      ENDIF
	   ELSEIF (Field Name .EQ. 'GR XORG   ') THEN
	      Xorigin = Temp    
	   ELSEIF (Field Name .EQ. 'GR YORG   ') THEN
	      Yorigin = Temp    
	   ELSEIF (Field Name .EQ. 'GR XCSIZE ') THEN
	      IF (Temp .GT. 0.0) THEN
	         XcellSize = Temp
	      ELSE
	         Error = .TRUE.
	         Message = 'X Cell size must be > 0 (press a key)'
	         Temp = XcellSize
	         Return Field = Field Name
	      ENDIF
	   ELSEIF (Field Name .EQ. 'GR YCSIZE ') THEN
	      IF (Temp .GT. 0.0) THEN
	         YcellSize = Temp
	      ELSE
	         Error = .TRUE.
	         Message = 'Y Cell size must be > 0 (press a key)'
	         Temp = YcellSize
	         Return Field = Field Name
	      ENDIF
	   ELSEIF (Field Name .EQ. 'GR XNCELL ') THEN
	      CALL Get Integer Value (Field Name, K)
	      IF (K .GT. 0 .AND. K .LE. 100) THEN
	         NxCells = K
	      ELSE
	         Error = .TRUE.
	Message = '# of cells must be > 0 and <=100 (press a key)'
	         CALL Set Integer Value (Field Name, NxCells)
	         Return Field = Field Name
	      ENDIF
	   ELSEIF (Field Name .EQ. 'GR YNCELL ') THEN
	      CALL Get Integer Value (Field Name, K)
	      IF (K .GT. 0 .AND. K .LE. 100) THEN
	         NyCells = K
	      ELSE
	         Error = .TRUE.
	Message = '# of cells must be > 0 and <=100 (press a key)'
	         CALL Set Integer Value (Field Name, NyCells)
	         Return Field = Field Name
	      ENDIF
	   ENDIF

C----------------------------------------------------------------------
C Get the kriging variable 
	ELSEIF (Field Name .EQ. 'KRIGEVAR  ') THEN
           KrigVar (Current Var) = Position + 1
	   CALL Display KrigList
	   IF (Ktype .EQ. SIMPLE) THEN
	       Return Field = 'VM MGLOBAL'
	   ELSE
	       Return Field = 'VM NUGGET '
	   ENDIF

C----------------------------------------------------------------------
C Get the number of digits to right of decimal pt.
C	ELSEIF (Field Name .EQ. 'VM NDIGITS') THEN
C           Ndigits (Current Var) = Position

C----------------------------------------------------------------------
C get the Model Parameters
	ELSE IF (Field Name .EQ. 'VM NUGGET ') THEN
	  IF (Temp .LT. 0.0) THEN
	     Error 	= .TRUE.
	     Message	= 'Nugget must be .GE. 0 - (press any key)'
	     Nugget (Current Var) = 0.0
	     Temp   	= 0.0
	     Return Field = Field Name
	  ELSE
	     Nugget (Current Var) = Temp
	  ENDIF 

	ELSEIF (Field Name .EQ. 'VM MGLOBAL') THEN
	  Mglobal (Current Var) = Temp

	ELSEIF (Field Name (1:2) .EQ. 'VM') THEN
	  READ (Field Name (10:10), '(I1)') Pos
	  IF (Field Name (4:7) .NE. 'TYPE' .AND.
     +		Mtype (Pos, Current Var) .EQ. 0) THEN
	        CALL Erase Field (Field Name)
	        Temp = 0.0
	  ELSE
	   IF      (Field Name (4:7) .EQ. 'TYPE') THEN
              Mtype (Pos, Current Var) = Position
              IF (Position .EQ. 0) then
                 CALL  Erase Structure (Pos)
                 if (pos .ne. maxstruc) then
                    Return Field = Field Name
                    write(Return Field(10:10),'(i1)') pos+1
                 else if (pos .eq. maxstruc) then
                    Return Field = 'SE MAXEMPT'
                 end if
              end if

	   ELSE IF (Field Name (4:7) .EQ. 'SILL') THEN
	    IF (Temp .LE. 0.0) THEN
	       Error 	= .TRUE.
	       Message 	=
     +          'This parameter must be .GT. 0 - (press any key)'
	       Temp 	= 0.0
	       Return Field = Field Name
	    ELSE
	       Sill (Pos, Current Var) = Temp
	    ENDIF 
	   ELSE IF (Field Name (4:9) .EQ. 'RMAJOR') THEN
	    IF (Temp .LE. 0.0) THEN
	       Error 	= .TRUE.
	       Message 	=
     +          'This parameter must be .GT. 0 - (press any key)'
	       Temp 	= 0.0
	       Return Field = Field Name
	    ELSE
	       VrMajor (Pos, Current Var) = Temp
	       IF (Character .EQ. 13) THEN
	         Dummy = FieldName (1:4)//'MINOR'//FieldName (10:10)
	         CALL Get Character Value (Dummy, Scratch)
	         IF (Scratch .EQ. '             ' .OR. 
     +	            VrMinor (Pos, Current Var) .GT. Temp) THEN
	            VrMinor (Pos, Current Var) = Temp
	            CALL Set 4Floating Point Value (Dummy, Temp, 3)
	         ENDIF 
	       ENDIF 
	    ENDIF 
	   ELSE IF (Field Name (4:9) .EQ. 'RMINOR') THEN
	    IF (Temp .LE. 0.0 .OR. Temp .GT.
     +	       VrMajor (Pos, Current Var)) THEN
	       Error 	= .TRUE.
	       Message 	=
     +          'Minor range must be in ]0. - Rmajor] (Press any key)'
	       Temp 	= VrMajor (Pos, Current Var)
	       Return Field = Field Name
	    ELSE
	       VrMinor (Pos, Current Var) = Temp
	    ENDIF 
	   ELSE IF (Field Name (4:8) .EQ. 'ANGLE') THEN
	    IF (Temp .LT. 0.0 .OR. Temp .GE. 180.0) THEN
	       Error 	= .TRUE.
	       Message	=
     +  'Angle must be .GE. 0  and .LT. 180 - (press any key)'
	       VrAngle (Pos, Current Var) = 0.0
	       Temp 	= 0.0
	       Return Field = Field Name
	    ELSE
	       VrAngle (Pos, Current Var) = Temp
	    ENDIF
	   ENDIF
	  ENDIF

	ELSE
	  CONTINUE
	ENDIF

C Set the real value at 3 decimal places
 80 	IF (Field Type .EQ. Real Field) THEN
	   IF (Pos .NE. 0 .AND.
     +	         Mtype (Pos, Current Var) .EQ. 0) THEN
	      CALL Set Character Value (Field Name, '             ')
	   ELSEIF (Field Name (1:8) .EQ. 'VM ANGLE') THEN
	     CALL Set 4 Floating Point Value (Field Name, Temp, 2)
	   ELSE
	     CALL Set 4 Floating Point Value (Field Name, Temp, 3)
	   ENDIF
	ENDIF

C write out field, in case an error caused it to be blanked out
	CALL Display One Field (Field Name)

C if an error occurred, put out the message
 90	IF (Error) CALL Flash (20, 2, Message)

C Set up Next Field to be processed
	Next Field = Fld Index (Return Field)
	RETURN
	END

C----------------------------------------------------------------------
C Erase Structure - erase a structure with Mtype = 0
C
	SUBROUTINE Erase Structure (Number)
	IMPLICIT INTEGER*4 (A-Z)

	INTEGER*4	Number
	CHARACTER	Field*10, Num*1, Fmt*4, Scratch*13

        INCLUDE 'krige.dec'

	DATA Fmt 	/'(I1)'/
	DATA Scratch 	/'             '/
	WRITE (Num, Fmt) Number

	Field = 'VM SILL  '//Num
	Sill (Number, Current Var) = 0.0
	CALL Set Character Value (Field, Scratch)
	CALL Display One Field (Field)

	Field = 'VM RMAJOR'//Num
	VrMajor (Number, Current Var) = 0.0
	CALL Set Character Value (Field, Scratch)
	CALL Display One Field (Field)

	Field = 'VM RMINOR'//Num
	VrMinor (Number, Current Var) = 0.0
	CALL Set Character Value (Field, Scratch)
	CALL Display One Field (Field)

	Field = 'VM ANGLE '//Num	
	VrAngle (Number, Current Var) = 0.0
	CALL Set Character Value (Field, Scratch)
	CALL Display One Field (Field)
	RETURN
	END	

