C----------------------------------------------------------------------
C PTLOC - returns the position of a point relative to a closed boundry
C
C       from Math Geology, Vol 7, No. 1, 1975 (John K. Hall)
C
C       Np      - Number of polygon points
C       X, Y    - Polygon X coordinates
C       Xp, Yp  - Coordinates of point whose location is to be tested
C  Loc, I1, I2  - Describe location of point Xp, Yp
C
C       LOC     = -1    The point lies outside the boundry
C       LOC     =  0     "       on the boundry between node I1, I2
C       LOC     = +1     "       inside the boundry
C       LOC     = +2     "       is on the node I1
C
C       Isr defines the sense of rotation of the boundry +1 = clockwise
C                                                        -1 = counter

        SUBROUTINE Ptloc (Np, X, Y, Xp, Yp, Loc, I1, I2, Isr)
        IMPLICIT INTEGER*4 (I-N)
        IMPLICIT REAL*4 (A-H)
        IMPLICIT REAL*4 (O-Z)

        REAL*4          X (N), Y (N), Xp, Yp
        INTEGER*4       Np, Loc, I1, I2, Isr

        REAL*4          Tsc, Sc, Tcc, Cc, X1, Y1, X2, Y2, R1, R2
        REAL*4          Vpl, St, Ct, C180
        INTEGER*4       Iec, Ip, I
        
        DATA            C180 /0.9999999998/
        I1      = 0
        I2      = 0
        Isr     = 0
        Iec     = 0
        Tsc     = 0.0
        Sc      = 0.0
        Tcc     = 1.0
        Cc      = 1.0
        Ip      = Np

        X1 = X (Np) - Xp
        Y1 = Y (Np) - Yp
        R1 = SQRT (X1 * X1 + Y1 * Y1)

C Check if point coincides with last polygon node
        IF (R1 .EQ. 0.0) THEN
              Loc = +2
              I1  = Ip
              I2  = Ip
              RETURN
        ENDIF
 
C Loop through all polygon nodes, evaluate the integrated function dtheta
        DO 45 I = 1, Np
           X2  = X (I) - Xp
           Y2  = Y (I) - Yp
           R2  = SQRT (X2 * X2 + Y2 * Y2)
           Vpl = R1 * R2

C Check if point coincides with polygon node
           IF (R2 .EQ. 0.0) THEN
              Loc = +2
              I1  = I
              I2  = I
              RETURN
           END IF

C calculate cross product and dot product for vector R1, and R2
           St  = (X1 * Y2 - X2 * Y1) / Vpl
           Ct  = (X1 * X2 + Y1 * Y2) / Vpl


C If Dtheta = 180 then point falls on the boundry between nodes I1,I2
           IF ((Ct + C180) .LE. 0.0) THEN
              Loc = 0
              I1  = Ip
              I2  = I
              RETURN
           END IF

C Check if I = Last node         
           IF (I .EQ. Np) THEN 
              IF (((Iec / 2) * 2 - Iec) .LT. 0) THEN 
                 Isr = +1
                 Loc = +1
                 IF (Sc .LT. 0.0) Isr = -1
                 RETURN
              ELSE
                 Loc = -1
                 RETURN
              ENDIF 
           ENDIF
C 
           Tsc = Sc * Ct + Cc * St
           Tcc = Cc * Ct - Sc * St
           IF ((Tsc*Sc .LE. 0.0) .AND. (Tsc*St .LT. 0.0)) Iec = Iec + 1
           Sc  = Tsc
           Cc  = Tcc
           X1  = X2
           Y1  = Y2
           R1  = R2
           Ip  = I
 45     CONTINUE
        END

                                             

                                                



