C
C Prevar.for - preprocessor for VARIO (creates a pair comparison file)
C
C----------------------------------------------------------------------
C Main +- Read Structure
C      |- Initialize ----------+- Set _______ Value
C      |                       +- Load Toggle Fields 
C      |		
C      |- Get Screen Fields ---+- Put Menu
C      | 		       |- Get Menu Choice
C      | 		       |- Compute Pairs - Qsort
C      | 		       |- Write Pairs
C      | 		       |- Get Field Group ------+
C      | 		       +- New Screen 		|
C      +- Clear Screen					|
C							|
C                   +-----------------------------------+
C  		    | 
C                   +->+- Get Field
C		       |- Perform Process +- Read Header Data 
C		       +- New Screen	  |- Read Coordinates
C					  |- Calc Possible Pairs
C 					  |- Get ________ Value/Position
C					  +- Set ________ Value/Position
C
C-----------------------------------------------------------------------
C
	PROGRAM Prevar

	INCLUDE 'prevar.dec'

        CALL Set up screen()
        CALL Initialize
        CALL Read System Defaults
        IF (Intro) CALL Intro Screen
        CALL Get Screen Fields
        CALL Write System Defaults
	END

C----------------------------------------------------------------------
C Intro Screen - 	displays screen frame/label/fields
C			for the introductory screen

	SUBROUTINE Intro Screen
	CALL Clear Screen
	CALL Set Screen (1)
	CALL Display Frame
	CALL Display Labels
cc adj - the following line was commented out and replaced by the
cc adj - line after it beccause the screen just aint that big.
cc adj - CALL Locate Cursor (25, 0)
        CALL Locate Cursor (23,0)
	CALL Flash (20, 2, 'Press any key to begin the program ')
	RETURN
	END

        
C----------------------------------------------------------------------
C Initialize - initialize PREVAR global values
C
	SUBROUTINE Initialize 
	IMPLICIT INTEGER*4 (A-Z)

	INCLUDE 'prevar.dec'

	DO 5 I = 1, MAXPAIR
	   PairPtr (I) = I
 5	CONTINUE

	DO 10 I = 1, MAXVAR
	   VarNam (I) = '          '
 10	CONTINUE
	CALL Load Toggle Values ('XVARIABLE ',10,MAXVAR,VarNam, 0)
	CALL Load Toggle Values ('YVARIABLE ',10,MAXVAR,VarNam, 1)

	Missing  = 1.E31

	NPair	= 0
	Nvar 	= 0
	Ndata	= 0
	Xvar	= 1
	Yvar	= 2

	Xmin		= +1.E30
	Xmax		= -1.E30
	Ymin		= +1.E30
	Ymax		= -1.E30
	MinDist		= +1.E30
	MaxDist		= -1.E30

	CALL Set Character Value ('DATAFILE  ', Data File)
	CALL Set Character Value ('PAIRFILE  ', Pair File)
	CALL Set Character Value ('XMIN      ', '         MIN')
	CALL Set Character Value ('XMAX      ',	'         MAX')
	CALL Set Character Value ('YMIN      ', '         MIN')
	CALL Set Character Value ('YMAX      ',	'         MAX')
	CALL Set Character Value ('MINDIST   ',	'         MIN')
	CALL Set Character Value ('MAXDIST   ',	'         MAX')
 
C Erase screen message area
 	   DO 444 I = 11, 18
	     CALL Erase (I, 42, 35)
 444 	   CONTINUE

C Set menu position to first
	CALL Set Menu Position ('MAINMENU  ', 1)
	RETURN
	END

C----------------------------------------------------------------------
C Get Screen Fields - 	displays screen frame/label/fields
C			gets the screen fields for the current screen
	SUBROUTINE Get Screen Fields 

	IMPLICIT INTEGER*4 (A-Z)
 
	INCLUDE 'prevar.dec'

	CHARACTER*10	Main Menu, Scratch
	CHARACTER*9	Menu Choice
	LOGICAL 	GetYesNo, Exit
	LOGICAL		Error

	DATA Main Menu	/'MAINMENU  '/

C Paint Screen
  5	CALL New Screen (2)
	CALL Set Character Value ('DATAFILE  ', Data File)

C Main menu processing	
 10 	CALL Put Menu (Main Menu)

 20	CALL Get Menu Choice (Main Menu, Menu Choice)	
 	IF      (Menu Choice .EQ. 'Quit     ') THEN
	  Exit = GetYesNo (20, 2, 'Do you really want to QUIT? <Y/N>')
	  IF (Exit) then
             call clear screen
             call reset screen
             RETURN
          end if
	ELSE IF (Menu Choice .EQ. 'Variables') THEN
	  IF (Nvar .EQ. 0) THEN
	     CALL Beep
	  ELSE
	     CALL Get Field Group
	  ENDIF

	ELSE IF (Menu Choice .EQ. 'Limits   ') THEN
	  IF (Ndata .EQ. 0) THEN
	     CALL Beep
	  ELSE
	     CALL Get Field Group
	  ENDIF

	ELSE IF (Menu Choice .EQ. 'Execute  ') THEN
	  IF (Nvar .EQ. 0 .OR. Ndata .EQ. 0) THEN
	     CALL Beep
	  ELSE
	    CALL Compute Pairs (Error)
	    IF (.NOT. Error) THEN
	       CALL Write Pairs	(Error)
	       IF (.NOT. Error) THEN
	          WRITE (Scratch,'(I10)') Npair
	          CALL Flash (20, 2, Scratch//
     +	     ' Pairs were written to the pair file (press any key)')
                  call clear screen
                  call reset screen
	          RETURN
	       ELSE
	          CALL Flash (20, 2,
     +       'Error creating pair file (press any key)')
	          NPair = 0
	       ENDIF
	       CALL Initialize
	       GOTO 5
	    ENDIF
	  ENDIF
	ELSE
	   CALL Get Field Group
	ENDIF
	GOTO 20
	END	

C----------------------------------------------------------------------
C Get Field Group - get a group of fields, perform processes

	SUBROUTINE Get Field Group 
	IMPLICIT INTEGER*4 (A-Z)

	PARAMETER (Main Menu Field = 3)
	PARAMETER (Menu Field      = 4)

	INCLUDE '../common/screen.str'
	
	LOGICAL		Error, Repaint

 10	CALL Get Field	
	CALL Perform Process (SC Field Name (Current Field),
     +                     Error, Repaint) 
	IF (SC Field Type (Next Field) .NE. Main Menu Field .AND.
     +	    SC Field Type (Next Field) .NE. Menu Field) GOTO 10

	IF (Repaint) THEN
	   CALL New Screen (Current Screen)
	ENDIF

	RETURN
	END

C----------------------------------------------------------------------
C New Screen (Screen number) - sets up a new screen
C
	SUBROUTINE New Screen (Screen Number)
	IMPLICIT INTEGER*4 (A-Z)

	INCLUDE '../common/screen.str'

	PARAMETER (Main Menu Field = 3)
	PARAMETER (Menu Field      = 4)

	CALL Clear Screen
	CALL Set Screen (Screen Number)
	CALL Display Frame
	CALL Display Labels

	DO 10 I = Begin Screen (Current Screen),
     +	         End Screen (Current Screen)
	  IF (SC Field Type (I) .NE. Main Menu Field .AND.
     +	      SC Field Type (I) .NE. Menu Field)
     +	      CALL Write Field (i, SC Attribute (i))

  10	CONTINUE
	RETURN
	END


C----------------------------------------------------------------------
C Read System Defaults

	SUBROUTINE Read System Defaults
	IMPLICIT INTEGER*4 (A-Z)
 
	INCLUDE 'prevar.dec'

	CHARACTER*20  String

	OPEN (UNIT = 10, FILE = 'geoeas.def', STATUS = 'unknown',
     +       ERR=999)
	READ (10, '(A)', ERR=999,END=999) File Prefix
	READ (10, '(A)', ERR=999,END=999) Data File
	READ (10, '(A)', ERR=999,END=999) String
	Intro = (String (7:8) .EQ. 'ON' .OR.
     +           String (7:8) .EQ. 'on')
	CALL Set Character Value ('FILEPREFIX', File Prefix)
	CLOSE (10)
	RETURN

 999	CLOSE (10)
	INTRO = .TRUE.
	RETURN
	END

C----------------------------------------------------------------------
C Write System Defaults

	SUBROUTINE Write System Defaults
	IMPLICIT INTEGER*4 (A-Z)
 
	INCLUDE 'prevar.dec'

	OPEN (UNIT = 10, FILE = 'geoeas.def', STATUS = 'unknown',
     +       ERR=999)
	WRITE (10, '(A)', ERR = 999) File Prefix
	WRITE (10, '(A)', ERR = 999) Data File
	IF (Intro) THEN
	  WRITE (10, '(A)', ERR = 999) 'INTRO=ON'
	ELSE
	  WRITE (10, '(A)', ERR = 999) 'INTRO=OFF'
	ENDIF
 999	CLOSE (10)
	RETURN
	END
C----------------------------------------------------------------------
C Perform Process - perform application-specific functions for PREVAR
C
	SUBROUTINE Perform Process (Field Name, Error, Repaint)
	IMPLICIT INTEGER*4 (A-Z)

	INCLUDE 'prevar.dec'
	INCLUDE '../common/screen.str'

	CHARACTER	Message*75, Blanks*14, OldFile*14, FileName*65
	CHARACTER*10	Field Name,  MainMenu, Dummy, Return Field
	INTEGER*4	Fld Index, Index of Field
	INTEGER*4	LENSTR, Position, I, J, K, L
	LOGICAL		Error, Exit, Repaint, PutMessage, FExist
        LOGICAL         GetYesNo, ReadXY
	REAL*4		Temp

	DATA MainMenu   /'MAINMENU  '/

C initialize flags
	Error		= .FALSE.
	Repaint		= .FALSE.	
	PutMessage	= .FALSE.
	FExist		= .FALSE.
	Index of Field	= Fld Index (Field Name)
	Return Field	= SC Field Name (Next Field)

C open data file, read in header information
	IF     (Field Name .EQ. 'FILEPREFIX') THEN
	  CALL Get Character Value (Field Name, File Prefix)
	  CALL Left Justify (File Prefix)
	  CALL Set Character Value (Field Name, File Prefix)
	ELSEIF (Field Name .EQ. 'DATAFILE  ') THEN
C	  OldFile = Data File
	  CALL Get Character Value (Field Name, Data File)
C	  IF (OldFile .NE. Data File) THEN
	     N = LENSTR (File Prefix)
	     IF (N .GT. 0) THEN 
	        File Name = File Prefix (1:N) // Data File
	     ELSE
	        File Name = Data File
	     ENDIF
	     INQUIRE (FILE = File Name, EXIST = Fexist)  
	     IF (.NOT. Fexist) THEN
	        Error = .TRUE.
	        Message = 'ERROR - data file not found'
                Return Field = Main Menu 
	        Data File = Blanks (1:14)
                CALL Set Character Value (Field Name, Data File)   
	     ELSE
	       Old File = Data File
	       CALL Initialize
	       Data File = Old File
	       CALL Set Character Value ('DATAFILE  ', Data File) 
	       CALL Display Fields
	       CALL Read Header Data (Error)
	       IF (Error) THEN
	         Message = 'ERROR - error reading data file'
                 CALL Set Character Value (Field Name, Blanks (1:14))   
                 Return Field = Main Menu 
	         Data File = Blanks (1:14)
                 CALL Set Character Value (Field Name, Data File)   
	       ELSE
	         CALL Read Coordinates (Error)
	         IF (Error) THEN
	            Message = 'ERROR - error reading data file'
                    Return Field = Main Menu 
	            Data File = Blanks (1:14)
                    CALL Set Character Value (Field Name, Data File)   
	         ELSE
	            L = LENSTR (Data File)
	            K = Index (Data File, '.')
	            IF (K .EQ. 0) THEN
	              Pair File = Data File (1:L) //'.pcf'
	            ELSE
	              Pair File = Data File (1:K-1) //'.pcf'
	            ENDIF
                    CALL Set Character Value ('PAIRFILE  ', Pair File)   
	            CALL Display One Field ('PAIRFILE  ')
                 ENDIF
               ENDIF
	    ENDIF
C	  ENDIF

C Get the pair comparison file name
	ELSEIF (Field Name .EQ. 'PAIRFILE  ') THEN
	  CALL Get Character Value (Field Name, Pair File)
	  N = LENSTR (File Prefix)
	  IF (N .GT. 0) THEN 
	     File Name = File Prefix (1:N) // Pair File
	  ELSE
	     File Name = Pair File
	  ENDIF
	  INQUIRE (FILE = File Name, EXIST = Fexist)  
          OverWrite = .TRUE. 
	  IF (Fexist) 
     + OverWrite = GetYesNo (20, 2, 'File Exists, Overwrite? <Y/N> ')
          IF (.NOT. Overwrite) THEN
	     Pair File = Blanks (1:14)
             CALL Set Character Value (Field Name, Pair File)   
	  ENDIF


C Get the coordinate variables
	ELSEIF (Field Name .EQ. 'XVARIABLE ') THEN
           CALL Get Toggle Value (Field Name, Position, Dummy)
           Xvar = Position + 1

C Check that X and Y variable are not the same, set Yvar
	ELSEIF (Field Name .EQ. 'YVARIABLE ') THEN
           CALL Get Toggle Value (Field Name, Position, Dummy)
           Yvar = Position + 1
	   IF (Xvar .EQ. Yvar) THEN
              Error   = .TRUE.
	      Message = 'ERROR - X and Y must be different variables '
              Return Field = 'XVARIABLE '
	   ELSE 
	     CALL Read Coordinates (Error)
	     IF (Error) THEN
	        Message = 'ERROR - error reading data file'
                CALL Set Character Value (Field Name, Blanks (1:14))   
                Return Field = Main Menu 
	        Data File = Blanks (1:14)
                CALL Set Character Value (Field Name, Data File)   
             ENDIF
	   ENDIF 

C limits
	ELSE IF (Field Name .EQ. 'XMIN      ') THEN
       	    CALL Get 4 Floating Point Value (Field Name, Xmin)
	 
	ELSE IF (Field Name .EQ. 'YMIN      ') THEN
       	    CALL Get 4 Floating Point Value (Field Name, Ymin)

	ELSE IF (Field Name .EQ. 'XMAX      ') THEN
       	    CALL Get 4 Floating Point Value (Field Name, Xmax)

 	ELSE IF (Field Name .EQ. 'YMAX      ') THEN
       	    CALL Get 4 Floating Point Value (Field Name, Ymax)

	ELSE IF (Field Name .EQ. 'MINDIST   ') THEN
       	    CALL Get 4 Floating Point Value (Field Name, MinDist)

	ELSE IF (Field Name .EQ. 'MAXDIST   ') THEN
       	    CALL Get 4 Floating Point Value (Field Name, MaxDist)
	    IF (Ndata .GT. 0) CALL Calc Possible Pairs
	ENDIF	

C if an error occurred, put out the message
	IF (Error) CALL Flash (20, 2, Message)

C write out field, in case an error caused it to be blanked out
	CALL Display One Field (Field Name)

C Set up Next Field to be processed
	Next Field = Fld Index (Return Field)

C Erase screen message area
 	   DO 444 I = 11, 18
	     CALL Erase (I, 42, 35)
 444 	   CONTINUE

	RETURN
	END


cc$LARGE
C Compute.for
C----------------------------------------------------------------------
C Compute Pairs - compute pair info, store pointer list

	SUBROUTINE Compute Pairs (Error)
	IMPLICIT INTEGER*4 (A-Z)
	
        INCLUDE 'prevar.dec'

	LOGICAL		Error
	LOGICAL 	GetYesNo, Recover
	CHARACTER	Scratch*10
	INTEGER*4	i, j, Isign, Zeropairs
	REAL*4		HXmin, Dir, Dh, Hx, Hy
	REAL*4		Tx1, Tx2, Ty1, Ty2, Dtr
        real*8 temp

	Recover 	= .false.
	ZeroPairs	= 0
 	Npair		= 0
	HXmin		= .0000001
	Dtr 		= 3.141592654 / 180.0

	CALL Putstr ('Calculating pair information...',31, 20, 2, 2)

	DO 10 i = 2, Ndata
	
	  Tx1 = X (i)
	  Ty1 = Y (i)

          IF (Tx1 .GE. Missing .OR. Ty1 .GE. Missing) GOTO 10

	  IF (Tx1 .LT. Xmin .OR. Tx1 .GT. Xmax) GOTO 10
	  IF (Ty1 .LT. Ymin .OR. Ty1 .GT. Ymax) GOTO 10

	  DO 20	j = 1, i-1
	    Tx2 = X (j)
 	    Ty2 = Y (j)
            IF (Tx2 .GE. Missing .OR. Ty2 .GE. Missing) GOTO 20
	    IF (Tx2 .LT. Xmin .OR. Tx2 .GT. Xmax) GOTO 20
	    IF (Ty2 .LT. Ymin .OR. Ty2 .GT. Ymax) GOTO 20
	    
c ***	calculate distance 
	    Hx	= Tx2 - Tx1
	    Hy  = Ty2 - Ty1

C ***	Zero distance pairs case
	    IF (Hx .EQ. 0. .AND. Hy .EQ. 0.) THEN
	       IF (MinDist .EQ. 0.0) ZeroPairs = ZeroPairs + 1
	       Dh = 0.0
	    ELSE
               temp = Hx * Hx + Hy * Hy
	       Dh = dSQRT (temp)
	    ENDIF

c ***	check IF distance is within distance limits
	    IF (Dh .GT. MaxDist .OR. Dh .LT. MinDist) GOTO 20

c ***	calculate Direction for pair
	    IF (ABS (Hx) .LT. HXmin) Hx = HXmin
	    Isign = INT (ABS (Hx) / Hx)		  
	    Dir	= atan(Hy/Hx) / Dtr
	    Dir	= (360.0 + Dir + (1 - FLOAT (Isign)) * 90.0)
	    IF (Dir .GT. 360.0) Dir = Dir - 360.0

c ***	store pair 
	    IF (Npair .le. MAXPAIR) THEN
	      Npair  		= Npair + 1
	      From (Npair)    	= i
	      To (Npair)      	= j
	      Distance (Npair)	= Dh
	      Direct (Npair)  	= Dir
              PairPtr (Npair) 	= Npair
	    ELSE
		Recover = Getyesno (20, 2, 
     +			'Max # pairs reached,  write to file? ') 
	    	IF (Recover) THEN
	          error = .false.
	          Npair = MAXPAIR
	          GOTO 999  
	        ELSE 
                  error = .true.
	          Npair = 0
	          GOTO 999
		ENDIF
	    ENDIF
 20	  CONTINUE
 10	CONTINUE

 999	CONTINUE
	IF (.NOT. Error) THEN
	   CALL Putstr (' Sorting pairs by distance... ',30, 20, 2, 2)
	   CALL Qsort  (Distance, PairPtr, Npair,maxpair)
	   IF (ZeroPairs .GT. 0) THEN
	      WRITE (Scratch, '(I10)') ZeroPairs
	      CALL Flash (20, 2, Scratch//
     +		' Zero Distance Pairs encountered ')
	   ENDIF  
	ENDIF
	CALL Erase (20, 2, 65)
	RETURN
	END

C----------------------------------------------------------------------
C Calc Possible Pairs - calculate possible pairs based upon limits
C
	SUBROUTINE Calc Possible Pairs
	IMPLICIT INTEGER*4 (A-Z)

        INCLUDE 'prevar.dec'

	CHARACTER*8	Scratch
	Nkept = 0
	DO 10 I = 1, Ndata
	   IF (X (I) .LT. Xmin .OR. X (I) .GT. Xmax .OR.
     +	       Y (I) .LT. Ymin .OR. Y (I) .GT. Ymax ) GOTO 10
	  Nkept = Nkept + 1
 10	CONTINUE
	WRITE (Scratch, '(I8)') Nkept * (Nkept-1) / 2
	CALL PutStr (Scratch, 8, 8, 58, 0)
	RETURN
	END

C----------------------------------------------------------------------
C ***	NONRECURSIVE QUICK SORT SUBROUTINE

       	SUBROUTINE QSORT (A, P, N, maxpair)

	IMPLICIT INTEGER*4 (A-Z)

	REAL*4 		A (maxpair), T,  T1
	INTEGER*4 	Tp, P (maxpair)
	DIMENSION 	LOWER(500),  UPPER(500)

		M	= 1
		I1	= 1
		J1	= N
		I	= I1
		J	= J1

 10		IF (I .GE. J) GOTO 110
 20		K = I
		I2 = INT ((J + I)/2)
C-------------------------------
		T = A(I2)
		TP = P(I2)
C-------------------------------
		IF (A(I) .LE. T) GOTO 30
C-------------------------------
		A(I2) = A(I)
		P(I2) = P(I)
		A(I) = T
		P(I) = TP
		T = A(I2)
		TP = P(I2)
C-------------------------------
 30		L = J
		IF (A(J) .GE. T) GOTO 40
C-------------------------------
		A(I2) = A(J)
		P(I2) = P(J)
		A(J) = T
		P(J) = TP
		T = A(I2)
		TP = P(I2)
C-------------------------------
		IF (A(I) .LE. T) GOTO 40
C-------------------------------
		A(I2) = A(I)
		P(I2) = P(I)
		A(I) = T
		P(I) = TP
		T = A(I2)
		TP = P(I2)
C-------------------------------
 40		L = L - 1
		IF (A(L) .GT. T) GOTO 40
C-------------------------------
		T1 = A(L)
		TP1 = P(L)
C-------------------------------
 50		K = K + 1
		IF (A(K) .LT. T) GOTO 50
		IF (K .GT. L) GOTO 60
C-------------------------------
		A(L) = A(K)
		P(L) = P(K)
		A(K) = T1
		P(K) = TP1
C--------------------------------
		GO TO 40

 60		IF (L-I .LE. J-K) GOTO 70
		LOWER(M) = I
		UPPER(M) = L
		I = K
		M = M+1
		GOTO 80
 70		LOWER(M) = K
		UPPER(M) = J
		J = L
		M = M+1
 80		IF (J-I .GE. I1) GOTO 20
		IF (I .EQ. I1) GOTO 10
		I = I-1
 90		I = I+1
		IF (I .EQ. J) GOTO 110
C-------------------------------
		T = A(I+1)
		TP = P(I+1)
C-------------------------------
		IF (A(I) .LE. T) GOTO 90
		K = I
C-------------------------------
 100		A(K+1) = A(K)
		P(K+1) = P(K)
C-------------------------------
		K = K-1
		IF (T .LT. A(K)) GOTO 100
C--------------------------------
		A(K+1) = T
		P(K+1) = TP
C--------------------------------
		GOTO 90
 110		M = M - 1
		IF (M .EQ. 0 ) GOTO 999
		I = LOWER(M)
		J = UPPER(M)
		GO TO 80
 999		RETURN
		END	




CC$LARGE
C Output.for - output subroutine for program PREVAR
C
C----------------------------------------------------------------------
C Write Pairs - write pair comparison file

	SUBROUTINE Write Pairs (Error)

	IMPLICIT INTEGER*4 (A-Z)

	INCLUDE 'prevar.dec'

	LOGICAL	Error
	CHARACTER*65	File Name, Dummy*80
	INTEGER*4	LENSTR
	REAL*4		DataRec (MAXVAR)

C Check for no pairs
	IF (Npair .EQ. 0) THEN
	    CALL Flash (20, 2, 'No pairs were computed (press a key)')
	    Error = .TRUE.
	    RETURN
	ENDIF

C Check for no file name
	CALL Left Justify (Pair File)
	IF (Pair File (1:1) .EQ. ' ') THEN
	   Error = .TRUE.
	   CALL Flash (20, 2,
     +       'No pair file name specified ... (press any key)')
	   RETURN
	ENDIF

C Begin
	CALL Putstr ('Writing to pair file...', 23, 20, 2, 2)

C Create and open output and data files 
	CALL Left Justify (File Prefix)
	N = LENSTR (File Prefix)
	IF (File Prefix (1:1) .NE. ' ') THEN 
	  File Name = File Prefix (1:N) // Pair File
	ELSE
	  File Name = Pair File
	ENDIF
        if (overwrite) then
           OPEN (UNIT = 10, FILE = File Name, STATUS = 'unknown',
     *          FORM = 'UNFORMATTED',ERR = 999)
        else
           OPEN (UNIT = 10, FILE = File Name, STATUS = 'new',
     *          FORM = 'UNFORMATTED',ERR = 999)
        end if           
	IF (File Prefix (1:1) .NE. ' ') THEN 
	  File Name = File Prefix (1:N) // Data File
	ELSE
	  File Name = Data File
	ENDIF
	OPEN (UNIT = 11, FILE = File Name, STATUS = 'OLD')
	READ (11, '(A)') Dummy
	READ (11, *) K
	DO 5 I = 1, K
	   READ (11, '(2A10)') Dummy, Dummy
 5	CONTINUE   

C Write file signature, header stuff
cc
cclong
	WRITE (10) '???prevar???'
cc This was changed from "prevar" to the ridiculous "???prevar???" because
cc that seems to be what Vario wants!  And it worked, too!
cc
	WRITE (10) data file, xvar, yvar, 
     +		   xmin, xmax, ymin, ymax, 
     +		   distmin, distmax

C WRITE number of variables, number of data
	WRITE(10, ERR = 999) Nvar, Ndata, Npair

C WRITE variable names and units to pair file
 	WRITE(10, ERR = 999) (varnam(i), i = 1, nvar)
	WRITE(10, ERR = 999) (units(i),  i = 1, nvar)

C write data matrix to pair file
	DO 10 I = 1, Ndata
	   READ  (11, *) 	    (DataRec (J), J = 1, Nvar)  
 	   WRITE (10, ERR = 999)    (DataRec (J), J = 1, Nvar)
 10	CONTINUE

C close data file
	CLOSE (11)

C WRITE pair comparison data to pair file
	WRITE (10, ERR = 999) (From     (PairPtr (i)), I = 1, Npair)
	WRITE (10, ERR = 999) (To       (PairPtr (i)), I = 1, Npair)
	WRITE (10, ERR = 999) (Distance          (i),  I = 1, Npair)
        WRITE (10, ERR = 999) (Direct   (PairPtr (i)), I = 1, Npair)

C close pair file
	CLOSE (10)
	Error = .FALSE.
	RETURN

 999	CONTINUE
	CLOSE (10)
	CLOSE (11)
	Error = .TRUE.
	RETURN
	END



CC$LARGE
C Readdata.for - file input routines for PREVAR
C
C----------------------------------------------------------------------
C ***  Read Header Data - get data from input file for program CONREC

	SUBROUTINE Read Header Data  (Error)
	IMPLICIT INTEGER*4 (A-Z)
 	
	INCLUDE 'prevar.dec'

	LOGICAL		Error
	CHARACTER*80	Header, Blanks*10, Scratch*5
	CHARACTER*65	File Name

	DATA Blanks /'          '/

C Build File name
	N = LENSTR (File Prefix)
	IF (N .GT. 0) THEN 
	  File Name = File Prefix (1:N) // Data File
	ELSE
	  File Name = Data File
	ENDIF

C Open File for input
	CALL PutStr ('Reading Header Data ',20, 20, 2, 2)
	Error 	= .FALSE.
	OPEN(UNIT = 10, FILE = File Name, STATUS ='OLD', 
     +       ERR = 999)

C Read header string into header character variable
	READ(10, '(A)', ERR = 999, END = 999) Header

C Get number of variables in file
	READ(10, *, ERR = 999, END = 999) nvar

	IF (Nvar .LT. 3) THEN 
	   CALL Flash (20, 2, 
     +              'Too few variables in file  (press any key)') 
	   GOTO 999
	ELSEIF (Nvar .GT. MAXVAR) THEN
	   CALL Flash (20, 2, 
     +              'Too many variables in file  (press any key)') 
	   GOTO 999
	ENDIF

C Get variable names for each variable in file
  	DO 20 I = 1,  Nvar
 	  READ (10,'(2A10)', ERR = 999, END = 999)
     +                                Varnam (i), Units (i)
 20     CONTINUE        

C Load variable names into toggle fields for screen processing
	CALL Load Toggle Values ('XVARIABLE ',10,Nvar,VarNam,Xvar-1)
	CALL Load Toggle Values ('YVARIABLE ',10,Nvar,VarNam,Yvar-1)

C display new field values
	CALL Display One Field  ('XVARIABLE ')
	CALL Display One Field  ('YVARIABLE ')

C display number of variables
	WRITE (Scratch, '(I5)') Nvar
	CALL PutStr (Scratch, 5, 6, 61, 0)
	Ndata 	= 0
	CLOSE (10)
	RETURN

C error has been found in data of file
 999	Error = .TRUE.
	CLOSE (10)
	CALL Beep
	CALL PutStr ('     ERROR reading data file ... ',33,11,43,2)
	CALL PutStr ('An ERROR was encountered in the  ',33,12,43,0)
	CALL PutStr ('header section of the data file. ',33,13,43,0)
	CALL PutStr ('Check the file name, and that the',33,14,43,0)
	CALL PutStr ('  named file has the the standard',33,15,43,0)
	CALL PutStr ('  data file format.              ',33,16,43,0)
        CALL PutStr ('Consult the Users Manual for more',33,17,43,0)
        CALL PutStr ('information about file format.   ',33,18,43,0)
	CALL Erase Field ('XVARIABLE ')
	CALL Erase Field ('YVARIABLE ')
	Ndata = 0
	Nvar  = 0
	Npair = 0
	RETURN
	END

C----------------------------------------------------------------------
C Read Coordinates - read coordinate data into memory

	SUBROUTINE Read Coordinates (Error)
	IMPLICIT INTEGER*4 (A-Z)
	
	INCLUDE '../common/screen.str'
	INCLUDE 'prevar.dec'

	CHARACTER 	Dummy*75, Scratch*8, File Name*65
cclong
        character       itch*7
cc  This character was introduced to take over some of Scratch's work (which 
cc wasn't being carried out too well).  The value of maxpair was extending
cc past its window.
cc
	LOGICAL		Error
	REAL*4		DataRec (MAXVAR), Dx, Dy
        real*8 temp

C Initialize Variables

	CALL PutStr ('Reading Coordinate Values', 25, 20, 2, 2)
	Ndata  		= 0

C Build File name
	N = LENSTR (File Prefix)
	IF (N .GT. 0) THEN 
	  File Name = File Prefix (1:N) // Data File
	ELSE
	  File Name = Data File
	ENDIF

C Open File for input
	Error 	= .FALSE.
	OPEN(UNIT = 10, FILE = File Name, STATUS ='OLD', 
     +       ERR = 999)

C Read header string 
	READ(10, '(A)', ERR = 999, END = 999) Dummy

C Get number of variables in file
	READ(10, *, ERR = 999, END = 999) Nvar

	IF (Nvar .LT. 1 .OR. Nvar .GT. MAXVAR) GOTO 999

C Get variable names for each variable in file
  	DO 20 I = 1,  Nvar
 	  READ (10,'(2A10)', ERR = 999, END = 999) Dummy, Dummy
 20     CONTINUE        

	Ndata  = 0
	Xmin   = +1.E30
	Ymin   = +1.E30
	Xmax   = -1.E30
	Ymax   = -1.E30
	MinDist = +1.E30
	MaxDist = -1.E30
	DO 30 Nrec = 1, MAXDATA
	   READ (10, *, END = 900, ERR = 999) (DataRec (i), i=1, Nvar)
	   Ndata = Ndata + 1
	   X (Ndata) = DataRec (Xvar)
	   Y (Ndata) = DataRec (Yvar)
	   IF (Xmin .GT. X (Ndata)) Xmin = X (Ndata)
	   IF (Ymin .GT. Y (Ndata)) Ymin = Y (Ndata)
	   IF (Xmax .LE. X (Ndata)) Xmax = X (Ndata)
	   IF (Ymax .LE. Y (Ndata)) Ymax = Y (Ndata)
 30	CONTINUE

C check if more than MAXDATA records
	READ (10, *, END = 900, ERR = 999) (DataRec (i), i=1, Nvar)
	CALL Beep
	CALL PutStr(' The number of records in the data ',35,11,42,2)
	CALL PutStr(' file exceeds the program limits.  ',35,12,42,2)
	CALL PutStr(' Only the maximum allowed number   ',35,13,42,0)
	CALL PutStr(' of coordinate values have been    ',35,14,42,0)
	CALL PutStr(' stored in memory, the rest will   ',35,15,42,0)
	CALL PutStr(' not be used ...                   ',35,16,42,0)
	CALL Flash (20, 2, 'Press any key to continue ')

 900	IF (Ndata .LE. 1) GOTO 999
 	CLOSE (10)
	Dx = Xmax - Xmin
	Dy = Ymax - Ymin
	IF (Dx .LE. 0. .AND. Dy .LE. 0.) GOTO 999
        temp = Dx*Dx + Dy*Dy
	MaxDist = dSQRT (temp)
	MinDist = 0.0
	CALL Set 4 Floating Point Value ('XMIN      ', Xmin, 3)
	CALL Display One Field          ('XMIN      ')
	CALL Set 4 Floating Point Value ('XMAX      ', Xmax, 3)
	CALL Display One Field          ('XMAX      ')
	CALL Set 4 Floating Point Value ('YMIN      ', Ymin, 3)
	CALL Display One Field          ('YMIN      ')
	CALL Set 4 Floating Point Value ('YMAX      ', Ymax, 3)
	CALL Display One Field          ('YMAX      ')
	CALL Set 4 Floating Point Value ('MINDIST   ', MinDist, 3)
	CALL Display One Field          ('MINDIST   ')
	CALL Set 4 Floating Point Value ('MAXDIST   ', MaxDist, 3)
	CALL Display One Field          ('MAXDIST   ')

	WRITE (Scratch, '(I8)') Ndata
	CALL PutStr (Scratch, 8, 7, 58, 0)
	K = Ndata * (Ndata-1) / 2
	WRITE (Scratch, '(I8)') K
	CALL PutStr (Scratch, 8, 8, 58, 0)
 	WRITE (itch, '(I7)') MAXPAIR
	CALL PutStr (itch, 7, 8, 72, 0)

	IF (K .GT. MAXPAIR) THEN
	  CALL PutStr('   # pairs possible > max limit    ',35,12,42,2)
	  CALL PutStr(' Unless limits are chosen for X, Y ',35,13,42,0)
	  CALL PutStr(' or interpair distance, too many   ',35,14,42,0)
	  CALL PutStr(' pairs will be computed.  The      ',35,15,42,0)
	  CALL PutStr(' maximum interpair distance will   ',35,16,42,0)
	  CALL PutStr(' be automatically reduced by 1/2...',35,17,42,0)
	  CALL Flash (20, 2, 'Press any key to continue ')
	  MaxDist = MaxDist * .5 
	  CALL Set 4 Floating Point Value ('MAXDIST   ', MaxDist, 3)
	  CALL Display One Field          ('MAXDIST   ')
	ELSE
	  CALL PutStr('Although the # pairs possible will ',35,11,42,0)
	  CALL PutStr('not exceed the program limitations,',35,12,42,0)
	  CALL PutStr('limits should be imposed on X, Y,  ',35,13,42,0)
	  CALL PutStr('or the interpair distances so that ',35,14,42,0)
	  CALL PutStr('the output file will not be too    ',35,15,42,0)
	  CALL PutStr('large.                             ',35,16,42,0)
	  CALL Flash (20, 2, 'Press any key to continue ')
	ENDIF
	RETURN

C error has been found in data
 999	Error 		= .TRUE.
	CLOSE (12)
	CLOSE (10)
	CALL Beep
	WRITE (Scratch, '(I5)') Ndata+Nvar+1
	CALL PutStr ('     ERROR reading data file ... ',33,11,43,2)
	CALL PutStr ('An ERROR was encountered reading ',33,12,43,0)
	CALL PutStr ('record #         in the data file',33,13,43,0)
	CALL Putstr (Scratch, 5, 13, 52, 2) 
	CALL PutStr ('Check the file name, and that the',33,14,43,0)
	CALL PutStr ('  named file has the the standard',33,15,43,0)
	CALL PutStr ('  data file format.              ',33,16,43,0)
        CALL PutStr ('Consult the Users Manual for more',33,17,43,0)
        CALL PutStr ('information about file format.   ',33,18,43,0)

        CALL putstr ('Type "c" to continue anyway:     ',33,20, 2,0)
        call getthekey(i)
        if (char(i) .eq. 'c' .or. char(i) .eq. 'C') then
           error = .false.
           CALL Erase (20, 2, 75)
           goto 900
        end if

	CALL Erase Field ('XVARIABLE ')
	CALL Erase Field ('YVARIABLE ')
	Ndata = 0
	Nvar  = 0
	Npair = 0
	CALL Erase (20, 2, 75)
	RETURN
	END



