cc$LARGE
C*********************************************************************
C*** File Name: PROCESS.FOR (Source code for the program SCATTER)
C*********************************************************************
C SUBROUTINE GET SCREEN FLD
C
C Local Variable:
C  Main Screen    - Stores the screen number for the main screen.
C  Main Menu      - Stores the name of the main menu.
C  Menu Choice    - Stores the user's menu selection.
C  Exit           - Logical variable that is set to true when the user
C                   wants to exit the program. 
C
C  This routine displays the screen, accesses the  menu option
C  selected by the user and then calls Perform Process to process
C  the screen fields or calls a routine to process the menu option. 
C
        SUBROUTINE Get Screen Fld
        
        IMPLICIT INTEGER*4 (A-Z)
   
        INCLUDE '../common/screen.str'
        INCLUDE 'scatter.cmn' 

        INTEGER*4      Main Screen
        CHARACTER*10   Main Menu
        CHARACTER*12   Menu Choice
   
        LOGICAL      GetYesNo, Exit

        DATA Main Menu  /'MAINMENU  '/
        DATA Main Screen     /2/
              
        Regression = .TRUE.
        True Scale = .FALSE.

C *** Display main menu
        CALL Get New Screen (Main Screen)
10      CALL Put Menu (Main Menu)
        CALL Display One Field ('PREFIX    ')

        CALL Get Menu Choice (Main Menu, Menu Choice)

C *** Quit - main menu
        IF (Menu Choice .EQ. 'Quit        ') THEN
           Exit = GetYesNo (20, 2, 'Do you really want to QUIT? <Y/N> ')
           IF (Exit)  THEN
              CALL Clear Screen
              CALL reset Screen
	      RETURN
	   ELSE
              GOTO 10
           ENDIF
        
C *** Data - main menu
        ELSE IF (Menu Choice .EQ. 'Data      ') THEN
           CALL Display One Field ('DATA FILE ')
           CALL Perform Process
           GOTO 10

C *** Prefix - main menu
        ELSE IF (Menu Choice .EQ. 'Prefix     ') THEN
           CALL Perform Process
           GOTO 10

C *** Execute - main menu
        ELSE IF (Menu Choice .EQ. 'Execute   ') THEN
           IF (Nvar .EQ. 0 .OR. Ndata .EQ. 0) THEN
              CALL Beep
              Text = 'ERROR - Variables not specified'
              CALL Flash (20,2,Text)
              GOTO 10
           ELSE
              CALL Scattergram (Title, Xlabel, Ylabel, Xarr, Yarr, 
     +             Ndata, Xmin, Xmax, Ymin, Ymax, 
     +             Regression, True Scale, DevCode,
     &             file prefix)
	      CALL Get New Screen (Main Screen)
	      CALL Display Flds
              GOTO 10
           ENDIF


C *** All other main menu options
        ELSE         
           IF (Nvar .EQ. 0 ) THEN
             CALL Beep
             Text = 'ERROR - Data file not specified'
             CALl Flash (20,2,Text)
             GOTO 10
           ELSE
              CALL Perform Process
              GOTO 10
           ENDIF
        ENDIF
        
	RETURN
        END

C***********************************************************************
C SUBROUTINE PERFORM PROCESS 
C
C Local Variables:
C  Field Name     - Stores the screen field name.
C  Fexist         - Logical variable set to true if the file exists.
C  Error          - Logical variable set to true if an error occurs.
C
C This routine identifies which screen field has been selected, performs
C any error checking and then processes the screen field.
C
        SUBROUTINE Perform Process 

        IMPLICIT INTEGER*4 (A-Z)

        INCLUDE '../common/screen.str'
        INCLUDE 'scatter.cmn'

        CHARACTER     Field Name*10, Blanks*50
        INTEGER*4     Fld Index
        LOGICAL     Fexist, Error

C *** Initialize flags
 	Error  = .FALSE.
        Fexist = .FALSE.  

C *** Identify fields, check for errors, perform process
  5     CALL Get Field
        Field Name = SC Field Name (Current Field)

C *** Set value for prefix (directory name)
	IF (Field Name .EQ. 'PREFIX    ') THEN
	   CALL Get Character Value (Field Name, File Prefix)

C *** open data file, read in header information
        ELSE IF (Field Name .EQ. 'DATA FILE ') THEN
           CALL Erase (6, 58, 5)
           CALL Erase (7, 58, 6)
           CALL Get Character Value (Field Name, Data File)
           IF (File Prefix .NE. Blanks) THEN
    	      la = Lenstr(File Prefix)
	      File Name = File Prefix(1:la)//Data File
           ELSE
	      File Name = Data File
	   ENDIF
	   INQUIRE (FILE = File Name,  EXIST = Fexist)
           IF (.NOT. Fexist) THEN
              Text = 'ERROR - data file not found'
              CALL Flash ( 20, 2, Text)
	      ERROR = .TRUE.
              CALL Erase Field (Field Name)
	   ELSE
              CALL Read Header Data (Error)
           ENDIF
	   Title = 'from data file '//Data file
           IF (.NOT. Error) THEN
              CALL Set Menu Position ('MAINMENU  ',5)
              Next Field = Fld Index ('XVARIABLE ')   
              GOTO 5
	   ENDIF
           
C *** Set value for variables  
   	ELSE IF (Field Name .EQ. 'XVARIABLE ') THEN 
           CALL Get Field Group
           CALL Get  XY Data
           IF (Nmissing .EQ. Ndata) THEN
              CALL Beep
              Text = 'ERROR - No data values.'
              CALL Flash (20,2,Text)
              Ndata = 0
              CALL Erase Field (Field Name)
              CALL Erase Field ('YVARIABLE ')
              RETURN
           ELSE
              IF (XLOG) THEN
                 l = Lenstr (Varnam (Xpos))
                 Xlabel = 'LN ('//Varnam(Xpos) (1:l) //')'
              ELSE
                 Xlabel = Varnam(Xpos)
              ENDIF
              IF (YLOG) THEN
                 l = Lenstr (Varnam (Ypos) )
                 Ylabel = 'LN ('//Varnam(Ypos) (1:l) //')'
              ELSE
                 Ylabel = Varnam(Ypos)
              ENDIF
           ENDIF
           CALL Erase (20,2,75)

C *** Set Regression option  
   	ELSE IF (Field Name .EQ. 'REGRESS    ') THEN
           CALL Get Field Group
           pos = Toggle Position (Field Name)
           IF (pos .EQ. 0) THEN
              Regression = .FALSE.
           ELSE
              Regression = .TRUE.
           ENDIF
           pos = Toggle Position ('TSCALE     ')
           IF (pos .EQ. 0) THEN
              True Scale = .FALSE.
           ELSE
              True Scale = .TRUE.
           ENDIF
        ELSE
           CALL Get Field
   	ENDIF

        RETURN
        END




