cc$LARGE
C*********************************************************************
C*** File Name: REGRESS.FOR (Source code for the program SCATTER)
C*********************************************************************
C SUBROUTINE REGRESS (XDATA, YDATA, NDATA, SLOPE, INTERCEPT,
C                     RSQUARED)
C
C Formal Arguements:
C  XDATA      -  Array of X data values.
C  YDATA      -  Array of Y data values.
C  SLOPE      -  Slope of the regression line.	
C  INTERCEPT  -  Y-intercept of the regression line.
C  RSQUARED   -  Stores the correlation coefficient.
C
C Local Variables:
C  N              - Number of data values.
C  Xbar, Ybar     - Mean of the X and Y data values.
C  Sumx, Sumy     - Sum of the X and Y data value. 	
C  SumXsq,SumYsq  - Sum of the X and Y data values squared.  	
C  SumXY   	  - Sum of the X and Y data values multiplied.
C
C This routine calculates linear regression. 
C        
        SUBROUTINE Regress (Xdata, Ydata, Ndata, Slope, Intercept,
     +                      RSquared)

        IMPLICIT INTEGER*4 (A-Z)

        REAL*4  Xdata(1), Ydata(1)
	REAL*4  X, Y, Xbar, Ybar, Missing
        REAL*4  Slope, Intercept, RSquared
        REAL*8 	Sxx, Syy, Sxy
        REAL*8  SumX, SumY, SumXsq, SumYsq, Sumxy
	
        LOGICAL Error

	DATA	Missing /1.E31/

	Error = .TRUE.       
	IF ((Ndata-Nmissing).LT. 2) RETURN

C *** Initialize
        N		= 0
        Xbar    	= 0.0
        Ybar    	= 0.0
        Sumx    	= 0.0
        Sumy    	= 0.0
        SumXsq  	= 0.0
        SumYsq  	= 0.0
        SumXY   	= 0.0
        Sxx     	= 0.0
        Syy     	= 0.0
        Sxy     	= 0.0
        RSquared 	= 0.0
        Slope	 	= 0.0
   	Intercept	= 0.0

	DO 10 I = 1, Ndata
           IF (Xdata (I) .GE. Missing .OR.
     +         Ydata (I) .GE. Missing) GOTO 10
           N 	     = N + 1
           X         = Xdata (I)
           Y         = Ydata (I) 
           SumX      = SumX + X
           SumY      = SumY + Y
           SumXsq    = SumXsq +  X * X
           SumYsq    = SumYsq +  Y * Y
           SumXY     = SumXY  +  X * Y
 10     CONTINUE 

C *** Calculate regression statistics        
        IF (N .GT. 0) THEN
	    Error 	  = .FALSE.
            Xbar          = SumX / N
            Ybar          = SumY / N
            Sxx           = SumXsq - (SumX * SumX) / N
            Syy           = SumYsq - (SumY * SumY) / N
            Sxy           = SumXY  - (SumX * SumY) / N
            Slope         = ((N*SumXY)  - (SumX*SumY))/
     +                      ((N*SumXsq) - (SumX*SumX)) 
            Intercept     = Ybar - Slope * Xbar
            RSquared      = Sxy / (SQRT (Sxx * Syy))
        ENDIF         

        RETURN
        END


