cc$LARGE
C 
C Compute.for - computation routine for STATS
C
C----------------------------------------------------------------------
C Calculate Stats
C----------------------------------------------------------------------
        SUBROUTINE Calculate Stats

        IMPLICIT INTEGER*4 (A-Z)

         INCLUDE 'stats.dec'

C Sx 		sum of data
C Sx2		sum squared
C Sx3		sum cubed
C Sx4		sum to the fourth power
C D		diffs between mean and individual values (squared, cubed, etc)
C Qlim 		Quartile cutoff indices
C SumW		Sum of weights
C CumFreq	Cumulative frequency
C Cum		""
C Vlast		temporary
C M1, M2, M3	temporary
C Interp	Interpolation function

        REAL*8          Sx, Sx2, Sx3, Sx4, D, D2, D3, D4, Temp
        REAL*4          Q1lim, Q2lim, Q3lim, W, Test, T
        REAL*4          Vlast, Cum, M2, M3, M4, SumW, CumFreq
        REAL*4          Interp, MeanWeight
        INTEGER*4       I, Index, Jvar, Nneg, Tptr
        CHARACTER       Number*4
C
C initialize values
C
        SumW            = 0.0
        Sx              = 0.0
        Sx2             = 0.0
        Sx3             = 0.0
        Sx4             = 0.0
        Mean            = 0.0
        Median          = 0.0
        Q1              = 0.0
        Q3              = 0.0
        Variance        = 0.0
        Stdev           = 0.0
        CoefVar         = 0.0
        IQrange         = 0.0
        Skewness        = 0.0
        Kurtosis        = 0.0
        WeightSum       = 0.0
	DataSum		= 0.0
        Nkept           = 0

	CALL PutStr ('Computing Statistics ... ', 25, 20, 2, 2)
C
C find first datum to use
C
cclong
 	FirstPtr = 1
 5	IF (Data (FirstPtr) .LT. MinValue .AND. 
     +            FirstPtr  .LT. Ndata) THEN
	   FirstPtr = FirstPtr + 1
	   GOTO 5
	ENDIF

	IF (FirstPtr .GE. Ndata - Nmiss) GOTO 20
C
C find last datum to use
C compute statistics: SumW, Min, Max, Sx
C
	Min = Data (FirstPtr)
	Max = Min
	LastPtr	= FirstPtr - 1

        DO 10 I = FirstPtr, Ndata - NMiss
	   IF (Data (I) .LE. MaxValue) THEN
	      LastPtr 	= I
	      Nkept 	= Nkept + 1
              SumW	= SumW + Weights (Ptr (I))
	      Sx	= Sx + Data (I) * Weights (Ptr (I))
	      IF (Min .GT. Data (I) ) Min = Data (I)
	      IF (Max .LT. Data (I) ) Max = Data (I)
	   ELSE
	      GOTO 20
	   ENDIF
 10     CONTINUE

C If no samples were retained, put out a message
 20     IF (Nkept .EQ. 0) THEN
           CALL Erase (20, 1, 78)
           CALL Flash (20, 2,
     +    'No Samples were retained - (check subsetting limits)')
           RETURN
        ENDIF
 
C Pass thru data, compute remaining values
C
	Weightsum  = SumW
	MeanWeight = 1.0
	IF (Weighted) MeanWeight = SumW / FLOAT (Nkept)
	IF (Max .GT. Min) THEN
           IF (SumW .NE. 0.0) Mean    = Sx / SumW
           Q1lim   = SumW * .25
           Q2lim   = SumW * .50
           Q3lim   = SumW * .75
           Cum     = 0.0
           Vlast   = Missing
	   CumFreq = 0.0
           DO  30 I = FirstPtr, LastPtr
              D    	= Data (I) - Mean
              W    	= Weights (Ptr (I))
              D2   	= D * D 
              D3   	= D * D2
              D4   	= D * D3
              Sx2  	= Sx2 + D2 * W 
              Sx3  	= Sx3 + D3 * W
              Sx4  	= Sx4 + D4 * W
	      CumFreq 	= CumFreq + W
 	      IF (SumW .GT. 0.0) Cpercent (I) = (CumFreq-W*.5) / SumW

C Determine interpolated quartile values
	      IF (Vlast .LT. Missing) THEN
                 IF ((Q1lim .GT. Cum) .AND. (Q1lim .LE. Cum + W)) THEN
                    Q1 =     Interp (Vlast, Data (I), Cum, W, Q1lim)
	         ENDIF
                 IF ((Q2lim .GT. Cum) .AND. (Q2lim .LE. Cum + W)) THEN
                    Median = Interp (Vlast, Data (I), Cum, W, Q2lim)
	         ENDIF
                 IF ((Q3lim .GT. Cum) .AND. (Q3lim .LE. Cum + W)) THEN
                    Q3 =     Interp (Vlast, Data (I), Cum, W, Q3lim)
	         ENDIF
	      ENDIF
              Vlast        = Data (I)
              Cum          = CumFreq
 30        CONTINUE

C
C median value
C
C	   IF (.NOT. Weighted) THEN
C	      Tptr = (FirstPtr+Nkept-1)/2
C	      IF (MOD (Nkept, 2) .EQ. 0) THEN
C		 Median = (Data (Tptr) + Data (Tptr+1))/2.0 
C	      ELSE
C		 Median = Data (Tptr+1)
C	      ENDIF
C	   ENDIF
 
C means, variances
C 
	   IF  (SumW .GT. MeanWeight)
     +               Variance = Sx2 / (SumW - MeanWeight)
	   IF (Variance .GT. 0.0) StDev    = SQRT (Variance)
	   IF (Mean .NE. 0.0) 	  CoefVar  = ABS (StDev/Mean)*100
	   IF (SumW .NE. 0.0) THEN
              M2              = Sx2 / SumW
              M3              = Sx3 / SumW
              M4              = Sx4 / SumW
              Skewness        = M3 / (M2 ** 1.5)
              Kurtosis        = M4 / (M2 * M2)
	   ENDIF
           IQrange      = Q3 - Q1
	ELSE
	   Mean 	= Min
	   Median 	= Min
	   Q1		= Min
	   Q3		= Min
	ENDIF
C
C computation finished, return
C
        RETURN
        END

C---------------------------------------------------------------------
C   Interpolation Function
           
        REAL*4 FUNCTION Interp (Y1, Y2, X1, Dx, X0)
        REAL*4  Y1, Y2, X1, Dx, X0
        REAL*8  U, Q

        U  = (X0 - X1) / Dx
        Q  = (1.0 - U) * Y1 + U * Y2
        Interp = Q
        RETURN
        END

                
                



