clarge
c**********************************************************************
c*** File name: QSORT.FOR (Source code for the program TRANS)
c**********************************************************************
c        R a n k  V a r                        (subroutine)
c**********************************************************************

      SUBROUTINE Rank Var(Col)

c*** This subroutine ranks the variable upon which the operation is performed
c*** from low to high.  The resulting variable stores this rank.  To find
c*** the rank of a variable's value refer to the corresponding row of the 
c*** result variable column.  That value will be the rank.
c*** 
c*** FORMAL ARGUMENT:
c*** Col     - Integer identifying the column number of variable in question.
c***
c*** VARIABLES:
c*** Ndata   - Global variable containing the number of sample records.
c*** Tmpdata - Global array containing ranking of samples for one variable.
c*** A1()    - Read array containing the samples of the variable specified by
c***           'Col'.
c*** P()     - an integer array which stores the original positions of 
c***           corresponding sample datum. 
c*** Nvar    - Global variable that stores the current number of variables.
c*** Missing - Global variable that stores the missing value.
c*** MAXVAR  - Global variable that contains the maximum number of variables
c***           permitted.

      include 'trans.dec'

      REAL*4     A1(MAXDATA), Get Element, Subtotal, Average
      INTEGER*4  P(MAXDATA), I, J, N, Countr
      LOGICAL  Vmiss, Same Values, EndofData

      N = Ndata
c***  Initialize the arrays.
      DO 10 I = 1, Ndata
        A1(I) = Get Element (1,Col,I,MAXVAR)
	P(I) = I
  10  CONTINUE

c***  Sort the specified variable's data stored in A1 in the ascending
c***  order.  Upon return A1 will contain the ordered sample data, and P 
c***  will contain the original position of that sample datum.  N is 
c***  assigned the value contained in 'Ndata' before this call. 

      CALL Qsort(A1, P, N)

      Countr = 0
c***  Assign ranks to array A1.
      DO 40 I = 1, Ndata
c***    Assign rank values to non-missing data.  Assign missing 
c***    values to the 'missing' data.
	IF (.NOT. (Vmiss(A1(I)))) THEN
	   Countr = Countr + 1
           Tmpdata(P(I)) = Countr
	ELSE
	   Tmpdata(P(I)) = Missing
	ENDIF
  40  CONTINUE

c***  Determine which samples are equal in value and then assign
c***  a revised rank based on the average value of their ranks.

      EndofData   = .FALSE.
      Same Values = .FALSE.
      Countr = 0
      Subtotal = 0.0
      DO 60 I = 1, Ndata - 1
	IF (Vmiss(A1(I))) RETURN
        IF (ABS(A1(I) - A1(I+1)) .LE. .000000001) THEN
          Same Values = .TRUE.
          Subtotal = Subtotal + Tmpdata(P(I))
          Countr = Countr + 1
          IF (I+1 .EQ. Ndata) THEN
            Subtotal = Subtotal + Tmpdata(P(Ndata))
            EndofData   = .TRUE.
          ENDIF
        ELSE
          IF (Same Values) THEN
            Subtotal = Subtotal + Tmpdata(P(I))
            Average = Subtotal/REAL(Countr+1)
            DO 50 J = I, I-Countr, -1
              Tmpdata(P(J)) = Average
  50        CONTINUE
            Countr = 0
            Subtotal = 0.0
            Same Values = .FALSE.
          ENDIF
        ENDIF
  60  CONTINUE

c***  If all variable's samples are equal then assign average.

      IF (Same Values .AND. (Countr .EQ. Ndata-1)) THEN
        Average = Subtotal/REAL(Countr+1)
        DO 70 J = 1, Ndata
          Tmpdata(P(J)) = Average
  70    CONTINUE

c***  Process those samples containing the maximum value.

      ELSEIF (Same Values .AND. EndofData) THEN
        Average = Subtotal/REAL(Countr+1)
        I = Ndata - Countr 
        DO 80 J = Ndata, I, -1
          Tmpdata(P(J)) = Average
  80    CONTINUE
      ENDIF

      RETURN
      END
