
C System.for
C----------------------------------------------------------------------
C Build System - set up system of equations for program XVALID
C
      SUBROUTINE Build System (X0, Y0, Neighbors, N,
     +				       TNeighbors, TN,  ZeroFlag)
	IMPLICIT INTEGER*4 (A-Z)

		include  'xvalid.dec'

C Passed Variables 
        REAL*4          X0, Y0, Gamma 
        INTEGER*4       Neighbors (1),  N, ZeroFlag
        INTEGER*4       TNeighbors (1), TN

C Local Variables
        INTEGER*4       I, J, IPtr, JPtr, Neq
        REAL*4          X, Y, Xc, Yc, D1x, D1y, D2x, D2y, G
        CHARACTER       Ch*1, stuff*60

C Initialize 
        ZeroFlag = 0
        DO 10 I = 1, MAXEQ
           B  (I)    = 0.0
           B2 (I)    = 0.0
           DO 10 J = 1, MAXEQ
               W (I, J) = 0.0
 10     CONTINUE        

C Build list of sample with non-missing values
	TN = 0
	DO 15 I = 1, N 
           IPtr         = Neighbors (I)
	   IF (Data (Iptr) .LT. Missing) THEN
	      TN = TN + 1
	      TNeighbors (TN) = Iptr
	   ENDIF
 15	CONTINUE
	IF (TN .LE. 0) RETURN

C Set up System of Equations for simple, or ordinary kriging
        DO 20 I = 1, TN
           IPtr         = TNeighbors (I)
           D1x          = XData (IPtr) - X0
           D1y          = YData (IPtr) - Y0
           B  (I)       = GamMax - Gamma (D1x, D1y)
           B2 (I)       = B (I)
           DO 25 J = 1, I-1
              JPtr      = TNeighbors (J)
              D2x       = Xdata (Iptr) - XData (Jptr)
              D2y       = YData (Iptr) - YData (Jptr)
              G         = Gamma (D2x, D2y)
C this code added to flag pairs with zero distance (duplicate pairs)
              IF (G .LE. 1.E-20) THEN
                W (I, J)  = GamMax * .999
                ZeroFlag  = ZeroFlag + 1
              ELSE
                W (I, J)  = GamMax - G
              ENDIF
	      W (J, I) = W (I, J)
 25        CONTINUE
	   W (I, I) = GamMax
 20	CONTINUE  

        Neq = TN

C Fill in non-bias conditions for ordinary Kriging  
        IF (Ktype .EQ. 0) THEN
            Neq = TN + 1
            DO 30 I = 1, TN
               W (TN+1, I) = 1.0
               W (I, TN+1) = 1.0
 30         CONTINUE
            W  (TN+1, TN+1) = 0.0
            B  (TN+1)       = 1.0
            B2 (TN+1)       = B (TN+1)
        ENDIF

	RETURN
        END

C----------------------------------------------------------------------
C Solve System - solve system of equations for program XVALID
C
        SUBROUTINE Solve System (Neighbors, N, Error)
	IMPLICIT INTEGER*4 (A-Z)

		include 'xvalid.dec'

        LOGICAL		Error
        INTEGER*4       Neighbors (1), N, Ndim
        INTEGER*4       K, I, Ptr, Index

        IF (Ktype .EQ. 1) THEN
           Ndim = N
        ELSE  
           Ndim = N + 1
        ENDIF

C Solve system of equations (LU - decomposition)
        CALL Simeq (Ndim, Error)


C Message for system error
        IF (Error) THEN
	   CALL Beep
cclong	   CALL Locate Cursor (24, 2)
	   CALL Locate Cursor (23, 2)
	   CALL Put Characters
     +	   ('Error in system of equations ...', 32, 0)
	   CALL getthekey (Key)
cclong	   CALL Erase (24, 2, 35)
	   CALL Erase (23, 2, 35)
	   Vk = Missing
	   Sk = Missing
           RETURN
        ENDIF

C compute kriging estimate and variance
        Vk = 0.0
        Sk = 0.0
        DO 10 K = 1, N
           Ptr = Neighbors (K)
           Vk = Vk + (Data (Ptr) - Mglobal) * B (K)
           Sk = Sk + B2 (K) * B (K)
 10     CONTINUE
        Vk = Vk  +  Mglobal
        Sk = GamMax - Sk - B (N+1)

C compute kriging standard deviation
	IF (Sk .GT. 0.) THEN
	    Sk = SQRT (Sk)
	ELSE
	   CALL Beep
cclong	   CALL Locate Cursor (24, 2)
	   CALL Locate Cursor (23, 2)
	   CALL Put Characters
     +	   ('Negative kriging variance ...', 29, 0)
C	   CALL getthekey (Key)
C	   CALL Erase (24, 2, 30)
	   Sk = Missing
	ENDIF  

        RETURN
        END

C----------------------------------------------------------------------
C Simeq - system of equations solver for program XVALID
C
        SUBROUTINE Simeq (N, Error)
	IMPLICIT INTEGER*4 (A-Z)

		include  'xvalid.dec'

C Passed variables
        INTEGER*4       N
        LOGICAL         Error

C Local variables
        INTEGER*4       I, J, K
        REAL*4          Tiny, Sum
        DATA            Tiny /1.0E-6/

C Solve system
        Error = .TRUE.
        DO 10 J = 1, N
           IF (J .GT. 1) THEN
              DO 20 I = 1, J-1
                 Sum =  W (I, J)
                 IF (I .GT. 1) THEN
                    DO 30 K = 1, I-1
 30                    Sum = Sum - W (I, K) * W (K, J)
                 END IF   
                 W (I, J) = Sum
 20           CONTINUE
           END IF

           DO 40 I = J, N
              Sum = W (I, J)
              IF (J .GT. 1) THEN
                 DO 50 K = 1, J-1
 50                 Sum = Sum - W (I, K) * W (K, J)
              ENDIF 
              W (I, J) = Sum
 40        CONTINUE

           IF (J .LT. N) THEN
              IF (W (J, J) .GE. Tiny) THEN
                 DO 60 I = J+1, N
                    W (I, J) = W (I, J) / W (J, J)      
 60              CONTINUE 
              ELSE
                RETURN 
              ENDIF
           END IF
 10     CONTINUE

        DO 70 I = 1, N
           Sum = B (I)
           IF (I .GT. 1) THEN
              DO 80 J = 1, I-1
 80              Sum = Sum - W (I, J) * B(J)
           ENDIF
           B (I) = Sum
 70     CONTINUE

        DO 90 I = N, 1, -1
           Sum = B (I)
           IF (I .LT. N) THEN
              DO 100 J = I+1, N
 100            Sum = Sum - W (I, J) * B (J)
           ENDIF
           B (I) = Sum / W (I, I)
 90     CONTINUE

        Error = .FALSE.
        RETURN
        END       
                  

C----------------------------------------------------------------------
C Gamma - function for calculating variogram value
C
        REAL*4  FUNCTION Gamma (Dx, Dy)
	IMPLICIT INTEGER*4 (A-Z)

		include 'xvalid.dec'

        REAL*4          Dx, Dy
	REAL*8		Power, Tiny, Teeny, Distance, Slope
        REAL*8          Dtr, Hu, Hv, G, R, H, Lambda, Angle

        DATA            Dtr 	/0.0174533/
	DATA		Tiny 	/1.E-6/
	DATA		Teeny 	/1.E-20/

C compute simple distance
	Distance = Dx * Dx + Dy * Dy
	IF (Distance .GT. 0.0) THEN
	   Distance = SQRT (Dx * Dx + Dy * Dy)
	ELSE
	   Distance = 0.0
	ENDIF

C variogram for 'zero' distance
	IF (Distance .LE. Teeny) THEN
	   Gamma = 0.0
	   RETURN

C variogram value for very small distances
	ELSE IF (Distance .LE. Tiny) THEN
           Gamma = Nugget
	   RETURN
	ENDIF
	
C variogram value for non-small distances
	G = Nugget
        DO 10 I = 1, 4
	   IF (Mtype (I) .EQ. 0) GOTO 10

C Compute rotated distance componments (Hu, Hv) (avoid COS/SIN cost)
	   IF (VrAngle (I) .GT. Tiny) THEN
             Angle      = - Vrangle (I) * Dtr
             Hu         = Dx * COS (Angle) - Dy * SIN (Angle)
             Hv         = Dy * COS (Angle) + Dx * SIN (Angle)
	   ELSE
	     Hu		= Dx 
	     Hv		= Dy
	   ENDIF

C Scale with anisotropy ratio and compute structural distance H 
           Hu           = Hu * (VrMinor (I) / VrMajor (I))
           H            = SQRT (Hu*Hu+Hv*Hv)

C Spherical
           IF      (Mtype (I) .EQ. 1) THEN
cclong: why is this minor instead of major?
              R = H / VrMinor (I)
cclong
              IF (R .GT. 1.0) R = 1.0
              G = G + Sill (I) * (1.5*R - 0.5*R*R*R)

C Gaussian (practical range is divided by SQRT (3))
           ELSE IF (Mtype (I) .EQ. 2) THEN
              R = H / (VrMajor (I) / 1.73205)
              G = G + Sill (I) * (1.-EXP(-R*R))

C Exponential (practical range is divided by 3)
           ELSE IF (Mtype (I) .EQ. 3) THEN
              R = H / (VrMajor (I) / 3.0)
              G = G + Sill (I) * (1.- EXP (-R))

C Linear
           ELSE IF (Mtype (I) .EQ. 4) THEN
	      Slope = Sill (I) / VrMajor (I)
              G = G + Slope * H
           ENDIF
 10     CONTINUE

        Gamma = G
        RETURN
        END


